/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.net.util;

 /*
  * Title: Query Cycle Simulator
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Message template.
  * @author unascribed
  * @version 1.0
  */

public class PeerMsg implements Cloneable
{

  public static final int ANY = Integer.MAX_VALUE;

  public static final int MAX_TTL = 5;
  public static final int MED_TTL = 3;
  public static final int MIN_TTL = 2;

  private int source_;

  private int destination_;

  private int ttl_;

  public PeerMsg()
  {
    source_      = 0;
    destination_ = 0;
    ttl_         = 0;
  }

  /**
   * Constructor
   * @param source Peer id of peer sending the message.
   * @param dest Peer id of destination.
   * @param ttl The message time to live.
   */
  public PeerMsg(int source, int dest, int ttl)
  {
    source_      = source;
    destination_ = dest;
    ttl_         = ttl;
  }

  public Object clone()
  {
    return new PeerMsg(source(), destination(), ttl());
  }

  public int hashCode()
  {
    return (source()*1000) + destination();
  }

  /**
   * The id of the peer that sent the message
   * @return peer id of the peer the sent the message.
   */
  public int source()
  {
    return source_;
  }

  /**
   * The peer id that this message is being sent to.
   * @return destination peer id.
   */
  public int destination()
  {
    return destination_;
  }

  /**
   * The time to live for this message
   * @return time to live.
   */
  public int ttl()
  {
    return ttl_;
  }

  /**
   * Set the time to live for this message
   * @param ttl The new time to live.
   */
  public void ttl(int ttl)
  {
    ttl_ = ttl;
  }

}
