/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */

package qcsim.net.util;

import java.io.*;

import qcsim.*;

 /**
  * Title: Query Cycle Simulator
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Message used to query peers for files.
  * @author unascribed
  * @version 1.0
  */

public class PeerMsgQuery extends PeerMsg implements Serializable
{
  /**
   * Content category containing the file being seeked
   */
  public int category_;

  /**
   * Rank of file being seeked
   */
  public int file_;

  /**
   * The connection that this message was used to send this message.
   */
  private int connection_;

  /** Constructor */
  public PeerMsgQuery() 
  {
    category_   = 0;
    file_       = 0;
    connection_ = 0;
  }

  /**
   * Constructor
   * @param source peer id sending the query
   * @param destination peer id sending the query
   * @param ttl time to live for the packet.
   * @param category content category containing the file requested.
   * @param file rank of file being requested.
   * @param connection The connection this message originated from.
   */
  public PeerMsgQuery(int source, int dest, int ttl, 
                      int category, int file, int connection) 
  {
    super(source, dest, ttl);

    category_   = category;
    file_       = file;
    connection_ = connection;
  }

  public boolean equals(Object obj)
  {
    if (obj instanceof PeerMsgQuery)
    {
      PeerMsgQuery pmq = (PeerMsgQuery) obj;
      if (pmq.source() == source() && pmq.destination() == destination())
      {
        if (pmq.category() == category() && pmq.file() == file())
          return true;
      }
    }
    return false;
  }
  
  /**
   * Copy Constructor
   * @param msg message to copy.
   */
  public Object clone() 
  {
    return new PeerMsgQuery(source(), destination(), ttl(),
                            category(), file(), connection());
  }

  public int category()
  {
    return category_;
  }

  public int file()
  {
    return file_;
  }

  public int connection()
  {
    return connection_;
  }

}
