/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */

package qcsim.net.util;


import java.io.*;

import qcsim.*;

 /**
  * Title: Query Cycle Simulator
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Message used to respond to a file query.
  * @author unascribed
  * @version 1.0
  */

public class PeerMsgQueryResponse extends PeerMsg
       implements Serializable 
{

  /**
   * Content category
   */
  public int category_;

  /** 
   * The fileRank. 
   */
  public int file_;

  /**
   * Connection used to send query.
   */
  public int connection_;

  public int responseTime_;

  /** Constructor */
  public PeerMsgQueryResponse() 
  {
    category_   = 0;
    file_       = 0;
    connection_ = 0;
  }

  /**
   * Constructor
   * @param source id of peer responding to query.
   * @param dest id of peer receiving this message.
   * @param category content category.
   * @param file the file rank.
   * @param connection the originating connection used to send the query.
   */
  public PeerMsgQueryResponse(int source, int destination,
                              int category, int file, int connection, int rt) 
  {
    super(source, destination, MAX_TTL);

    category_   = category;
    file_       = file;
    connection_ = connection;
    responseTime_ = rt;
  }

  /**
   * Constructor
   * @param source id of peer responding to query.
   * @param dest id of peer receiving this message.
   & @param ttl time to live.
   * @param category content category.
   * @param file the file rank.
   * @param connection the originating connection used to send the query.
   */
  public PeerMsgQueryResponse(int source, int destination, int ttl,
                              int category, int file, int connection, int rt) 
  {
    super(source, destination, ttl);   

    category_   = category;
    file_       = file;
    connection_ = connection;
    responseTime_ = rt;
  }

  public boolean equals(Object obj)
  {
    if (obj instanceof PeerMsgQueryResponse)
    {
      PeerMsgQueryResponse pmqr = (PeerMsgQueryResponse) obj;
      if (pmqr.source() == source() && pmqr.destination() == destination())
      {
        if (pmqr.category() == category() && pmqr.file() == file())
          return true;
      }
    }
    return false;
  }

  public Object clone() 
  {
    return new PeerMsgQueryResponse(source(), destination(), ttl(), 
                                    category(), file(), connection(), 
                                    responseTime());
  }

  public String toString()
  {
    return "PeerMsgQueryResponse: source "+source()+
           " destination "+destination();
  }

  /**
   * Content category containing the requested file.
   * @return the content category.
   */
  public int category()
  {
    return category_;
  }

  /**
   * The file rank requested by the peer.
   * @return the file rank.
   */
  public int file()
  {
    return file_;
  }

  public int connection()
  {
    return connection_;
  }

  public int responseTime()
  {
    return responseTime_;
  }

}
