/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */

package qcsim.net.util;

 /*
  * Title: Query Cycle Simulator
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /** 
  * Provides an interface to simulate the upload of a file. 
  * @author unascribed
  * @version 1.0
  */

public class Upload 
{
  /**
   * peer ID of downloading peer
   */
  int downloadPeerID_;
  /**
   * transfer rate of upload
   */
  double transferRate_;
  /**
   * kbytes left to be transferred
   */
  double kbyteLeft_;
  
  /**
   * Constructor
   * @param downloadPeerID peer ID of downloading peer
   * @param transferRate transfer rate of upload
   * @param kbyteLeft kbytes left to be transferred
   */
  public Upload(int downloadPeerID, double transferRate, 
                      double kbyteLeft) 
  {
    downloadPeerID_ = downloadPeerID;
    transferRate_   = transferRate;
    kbyteLeft_      = kbyteLeft;
  }
  
  /**
   * Set the transfer rate
   * @param rate the transfer rate
   */
  public void setTransferRate(double rate) 
  {
    transferRate_ = rate;
  }
  
  /**
   * Simulates upload for some seconds
   * @param numSec  number of seconds to be simulated
   * @return true if download is finished
   */
  public boolean upload(int numSec) 
  {
    kbyteLeft_ -= numSec * transferRate_;
    return kbyteLeft_ < 0;
  }
}
