/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.peer;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;



 /*
  * Title: Query Cycle Simulator
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Provides a GUI description of a peer and its downloads.
  * @author unascribed
  * @version 1.0
  */

public interface PeerDescription
{
  /**
   * This method will set display information needed by
   * the peer's description object.
   * @param dim describes the display dimension.
   * @param pos used to conpute the peer's position on the display
   */
  public void displayDescription(Dimension dim, double pos);

  /**
   * This will draw the peer with the specified color.
   * Use the method color to get a color based on the peers
   * behavior or current trust value.
   * @param g The graphics to draw on.
   * @param color The color of the peer.
   */
  public void draw(Graphics g, Color color);

  /**
   * Get a descriptions of the peers downloads.
   * @return A vector of download descriptions.
   */
  public Vector downloadDescriptions();

  /**
   * This will draw the links that connect this peer to others.
   * @param color The color of the links.
   */
  public void drawLinks(Graphics g);

  /**
   * Funtion will return the color that the peer should use to paint itself.
   * @param trust If true then base color off of trust value.
   */
  public Color color(boolean trust);

  /**
   * Will specify if this peer is contains the given coordinates.
   * @param x the x position.
   * @param y the y position.
   * @return true if peer description contains the coordinates.
   */
  public boolean contains(int x, int y);

  /**
   * The current position of the peer description on the display.
   * @return the center point of the peer's position.
   */
  public Point2D position();

  public void move(PeerDescription neighbor, boolean away);
}
