/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.peer.content;

import java.util.*;
import java.io.*;

 /*
  * Title: Query Cycle Simulator
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * This interface represents a given content category.  It contains
  * a files associated with the content category and is given to
  * a peer as part of its shared data.
  * @author unascribed
  * @version 1.0
  */

public interface Content
{
  public static final double BAD_FILE  = 0.0;
  public static final double GOOD_FILE = 1.0;

  /**
   * The content category id.
   * @return category id.
   */
  public int category();

  /**
   * The peers interest level in this content category.
   */
  public double interest();

  /**
   * Set the peers interest level in this content category.
   */
  public void interest(double interest);

  /**
   * The total volume of data (number of files) 
   * in this content category.
   */
  public int volume();

  /**
   * Set the total volume of data (number of files)
   * in this content category
   */
  public void volume(int volume);

  /**
   * The number of files.
   * @return The number of files.
   */
  public int files();

  public int file(int i);

  /**
   * Search for the file rank in this content category
   * @param rank the file to search.
   * @return whether the content contains the file.
   */
  public boolean containsFile(int rank);

  /**
   * Add the file rank to this content category
   * @param rank file to add.
   * @return true if file was successfully added.
   */
  public boolean addFile(int rank);
}
