/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.peer.content;

import java.util.*;
import java.io.*;

 /*
  * Query Cycle Simulator
  * P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /** 
  * Interface to a peer's content.
  * @author unascribed
  * @version 1.0
  */

public interface PeerContent
{

  /**
   * The manager of this PeerContent object.
   * @return The Peer Content Manager.
   */
  public PeerContentManager manager();

  /**
   * Array containing the Peer's content distribution.
   * @return array with content distribution
   */
  public double[] contentDistribution();

  /**
   * The total amount of volume this peer shares.
   * @return total volume across all content categories.
   */
  public int totalVolume();
  public int totalFiles();

  /**
   * The number of content categories this peer contains.
   * @return the number of content categories.
   */
  public int categories();

  /**
   * Get the content category
   * @param selector content category selector.
   */
  public Content category(int index);

  public Content findCategory(int category);

  /**
   * Choose a random category to download from
   * @return the category
   */
  public int randomCategory();

  /**
   * Choose a random file to download.  This will not pick a file already owned.
   * It will also prefer highly ranked files.
   * @param category choose file from this content category
   * @param search If true will ensure the random file exists in the network.
   * @return the file choosen.
   */
  public int randomFile(int category);

}


