/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.peer.content;

import java.util.*;
import java.io.*;

 /*
  * Query Cycle Simulator
  * P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /** 
  * Represents the total content of a given peer.
  * @author unascribed
  * @version 1.0
  */

public interface PeerContentManager 
{
  /** data volume distribution models */
  public static final int SAROIU        = 0;
  public static final int UNIFORMRANDOM = 1;

  /** Category preference */
  public static final int TOP_CATEGORY    = 0;
  public static final int BOTTOM_CATEGORY = 1;
  public static final int RANDOM_CATEGORY = 2;

  /**
   * The total number of content categories in the system.
   * @return total number of content categories.
   */
  public int contentCategories();

  /**
   * The number of peer content objects
   * @return total number of peer content objects
   */
  public int peerContents();

  /**
   * Get the nth peer content.
   * @param index index to the peer content.
   * @return The peer content.
   */
  public PeerContent peerContents(int index);

  /**
   * Get the category normalization factor.
   * @return category normalization factor.
   */
  public double normFactor(int category);

  /**
   * Get the number of unique files in category
   * @param category The Content category.
   * @return The number of unique files in the category.
   */
  public int uniqueFiles(int category);

    /**
     * Gets the popularity of a file or category
     * popularity is defined as the probability that a file drawn randomly
     * from the union of all users' libraries matches the file/category
     */
    public double getFilePopularity(int category, int file);
    public double getCategoryPopularity(int category);
    public int getTotalNumFiles();

    /**
     *  Take note as files are replicated/downloaded.
     */
    public void addFileCategoryCount(int category, int file);

  /**
   * Fuction will initialize a peer's content by assigning a number of 
   * content category and a giving each content category a level of interest
   * @param categoryPreference Specified category selection preference.
   */
  public PeerContent peerContent(int categoryPreference);

  /**
   * This should be called once all peer content objects have been
   * created.  This will then initialze the peer content objects with
   * actual files.
   */
  public void distributeFiles();

}


