/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.peer.impl;

import java.util.*;
import java.io.*;

import qcsim.net.util.PeerMsg;
import qcsim.QueryCycleManager;
import qcsim.peer.*;

 /**
  * Creates and manage all peers in a simulation.
  *
  * @author unascribed
  * @version 1.0
  */

public class PeerManagerRTR extends PeerManagerImpl
{
    public static final int PROFIT_SELLER = 0;
    public static final int PROFIT_BUYER  = 1;
    public static final int PROFIT_BOTH   = 2;

    // some temporary statistics
    public int numOffersTTL[];
    public int numOffers;
    public int numFilterRejects;
    public int numPriceRejects;
    public int numDuplicateRejects;
    public int numDuplicateRejectsTTL[];
    public int numApproxGuesses, numExactGuesses;
    public double totalApproxDiff, totalExactDiff;

  public int profitType_;

  public boolean evolution_;

  public PeerManagerRTR(QueryCycleManager manager) {
	super(manager);
          profitType_ = PROFIT_BOTH;
          evolution_ = false;
	numOffers = numDuplicateRejects = numFilterRejects =
	    numPriceRejects = numApproxGuesses = numExactGuesses = 0;
	totalApproxDiff = totalExactDiff = 0; 
	numOffersTTL           = new int[PeerMsg.MAX_TTL+1];
	numDuplicateRejectsTTL = new int[PeerMsg.MAX_TTL+1];
	for (int i=0; i <= PeerMsg.MAX_TTL; i++) {
	    numOffersTTL[i] = numDuplicateRejectsTTL[i] = 0;
	}
    }

    public void outputStats() {
	System.out.println(numOffers + "\tNum Offers");
	System.out.println(numFilterRejects + "\tFilter Rejects");
	System.out.println(numPriceRejects + "\tPrice Rejects");
	System.out.println(numDuplicateRejects + "\tDuplicate Rejects");

	for (int i=0; i < PeerMsg.MAX_TTL; i++) {
	    System.out.print((1 - (double)numDuplicateRejectsTTL[i]/
			      numOffersTTL[i]) + " ");
	}
	System.out.println(" probbuy");
	System.out.println((totalApproxDiff/numApproxGuesses) + " average approx diff");
	System.out.println((totalExactDiff/numExactGuesses) + " average exact diff");
    }

  public void outputStats(FileOutputStream out)
  {
    PeerBehaviorRTR b;
    String          row;


    if (manager().cycle() == 0)
    {
      b = (PeerBehaviorRTR) peer(0).behavior();

      row = "cycle peerID peerType filterLevel fracReturn ";
      for (int j = 0; j < b.resellFudgeFactors(); j++)
        row += "ResellFudge" + j + " ";
      for (int j = 0; j < b.buyFudgeFactors(); j++)
        row += "BuyFudge" + j + " ";
      row += "VoidDownloads Uploads Volume Neighbors ";
      row += "\n";

      try {
        out.write( row.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }

    for (int i = 0; i < peers(); i++)
    {
      b = (PeerBehaviorRTR) peer(i).behavior();

      row = "";
      row += manager().cycle() + " " + peer(i).peerID() + " ";
      row += b.type() + " ";
      row += b.filterLevel() + " " + b.fracReturn() + " ";
      for (int j = 0; j < b.resellFudgeFactors(); j++)
        row += b.resellFudgeFactor(j) + " ";

      for (int j = 0; j < b.buyFudgeFactors(); j++)
        row += b.buyFudgeFactor(j) + " ";
      row += peer(i).voidDownloads() + " ";
      row += peer(i).totalGoodUploads() + " ";
      row += peer(i).content().totalVolume() + " ";
      row += peer(i).neighbors() + " ";
      row += "\n";

      try {
        out.write( row.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  public int profitType()
  {
    return profitType_;
  }

  public void profitType(int profitType)
  {
    profitType_ = profitType;
  }

  public boolean evolution()
  {
    return evolution_;
  }

  public void evolution(boolean evolution)
  {
    evolution_ = evolution;
  }

    // ---------------------------
    // Functions for plots
  /**
   * Create a new peer
   * @param behavior the peer's behavior/type
   * @return the peer.
   */
  public Peer createPeer(int peerID, int behavior) {
    return new PeerRTR(this, peerID, behavior);
  }

  public void rtrType(int rtrType)
  {
    for (int p = 0; p < peers(); p++) 
      ((PeerBehaviorRTR) (peer(p).behavior())).type(rtrType);
  }

  public void filterLevel(Vector levels, Vector dist)
  {
    PeerBehaviorRTR b = null;

    int pid = 0;
    while (pid < peers())
    {
      for (int i = 0; i < levels.size(); i++)
      {
        int filter = Integer.parseInt(((String)levels.elementAt(i)).trim());
        int npeers = Integer.parseInt(((String)dist.elementAt(i)).trim());
        for (int j = 0; j < npeers && pid < peers(); j++, pid++) 
        {
          b = (PeerBehaviorRTR) peer(pid).behavior();
          b.filterLevel(filter);
        }
      }
    }
  }

  public void fudgeFactor(int num, Vector fValues, Vector dist)
  {
    PeerBehaviorRTR b   = null;
    int             pid = 0;

    while (pid < peers())
    {
      for (int i = 0; i < dist.size(); i++)
      {
        int      npeers = Integer.parseInt(((String)dist.elementAt(i)).trim());
        double[] fudge  = new double[num];

        for (int f = 0; f < num; f++)
        {
          int index = (i * num) + f;
          fudge[f] = Double.parseDouble(((String)fValues.elementAt(index)).trim());
        }

        for (int p = 0; p < npeers && pid < peers(); p++, pid++) 
        {
          b = (PeerBehaviorRTR) peer(pid).behavior();
          b.resellFudgeFactor(fudge);
        }
      }
    }
  }

  public void probBuy(Vector probBuy)
  {
    PeerBehaviorRTR b = null;
    double[] pb  = new double[probBuy.size()];

    for (int i = 0; i < probBuy.size(); i++)
      pb[i] = Double.parseDouble(((String)probBuy.elementAt(i)).trim());   

    for (int p = 0; p < peers(); p++) 
    {
      b = (PeerBehaviorRTR) peer(p).behavior();
      b.probBuy(pb);
    }
  }

  public void fracReturn(double fracReturn)
  {
    for (int p = 0; p < peers(); p++) 
      ((PeerBehaviorRTR) (peer(p).behavior())).fracReturn(fracReturn);
  }

  public void bargainFactor(double bf)
  {
    /*
    for (int p = 0; p < peers(); p++) 
      ((PeerBehaviorRTR) (peer(p).behavior())).bargainFactor(fracReturn);
    */

  }

  public void execCycle()
  {
      if (manager().cycle() == 54)
	  System.out.println("hi");

    for (int i = 0; i < peers(); i++)
      ((PeerRTR)peer(i)).newCycle();

    for (int i = 0; i < peers(); i++)
      ((PeerRTR)peer(i)).query();

    // do steps within each query, so that ttls are spread more uniformly
    for (int f=0; f < PeerMsg.MAX_TTL-1; f++) {
	System.out.println("STEP " + f);
	for (int i=0; i < peers(); i++) 
	    ((PeerRTR)peer(i)).initStep();
	for (int i=0; i < peers(); i++) 
	    ((PeerRTR)peer(i)).step();
    }

    for (int i = 0; i < peers(); i++)
      ((PeerRTR)peer(i)).endCycle();

    for (int i = 0; i < 25; i++)
    {
      for (int j = 0; j < peers(); j++)
        ((PeerRTR)peer(j)).preSubmitTrust(i);
      for (int j = 0; j < peers(); j++)
        ((PeerRTR)peer(j)).submitTrust();
      for (int j = 0; j < peers(); j++)
        ((PeerRTR)peer(j)).postSubmitTrust();
    }
  }

}
