/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.peer.impl;

import java.util.Vector;
import java.util.Random;
import java.util.ArrayList;
import java.util.Enumeration;

import qcsim.peer.*;
import qcsim.peer.content.*;
import qcsim.net.*;
import qcsim.net.util.*;

 /*
  * Query Cycle Simulator
  * P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * This object contains all information associated with a peer in
  * the network.
  * @author unascribed
  * @version 1.0
  */

public class PeerRTR extends PeerImpl
{

    // The peer's current bank account balance
    protected double balance_;

    // The new RTRs received from this step
    protected Vector newRTR_;

    // The new RTRs from the previous step
    protected Vector prevRTR_;

    // current query information
    protected boolean bHasQuery;
    protected int file_;
    protected int category_;

    // datamembers for plots
    protected Vector totalScores_;
    protected Vector exactMatches_;

    // keep track of peers who have queried for related content in the
    // past
    protected Vector interestedPeers_;

    /**
     * Constructor.
     * @param manager The manager that created this peer.
     * @param peerID peer ID
     * @param behavior The peer's behavior.
     */
    public PeerRTR(PeerManager manager, int peerID, int behavior)
    {
	super(manager, peerID, behavior);
	behavior_ = new PeerBehaviorRTR(this, behavior);
	balance_ = 10;
	newRTR_ = new Vector();
	prevRTR_ = new Vector();

	exactMatches_ = new Vector();
	totalScores_ = new Vector();
	interestedPeers_ = new Vector();

	exactMatches_.addElement( new Integer(0) );
	totalScores_.addElement( new Double(0) );
    }

    /**
     * Constructor.
     * @param manager The manager that created this peer.
     * @param peerID peer ID
     * @param behavior The peer's behavior.
     */
    public PeerRTR(int debug, PeerManager manager, int peerID, int behavior)
    {
	super(debug, manager, peerID, behavior);
	behavior_ = new PeerBehaviorRTR(this, behavior);
	balance_ = 0;
	newRTR_ = new Vector();
	prevRTR_ = new Vector();

 	exactMatches_ = new Vector();
	totalScores_ = new Vector();
	interestedPeers_ = new Vector();

	exactMatches_.addElement( new Integer(0) );
	totalScores_.addElement( new Double(0) );
   }


   public double balance() { return balance_;}
   public void balance(double b) {balance_ = b;}

   protected RevenueTracker getRevenueTracker(int id) {
	Integer it = new Integer(id);
	RevenueTracker rt =  (RevenueTracker) connectionOpinions_.get(it);

	if (rt == null) {
	    rt = new RevenueTracker();
	    connectionOpinions_.put(it, rt);
	}
	return rt;
    }

    protected void removeRevenueTracker(int id) {
	Integer it = new Integer(id);
	connectionOpinions_.remove(it);
    }

    protected double getProfit(int id) {
      int pType = ((PeerManagerRTR)manager()).profitType();
	return getRevenueTracker(id).getProfit(pType);
    }

//=============================================================

    protected Peer pickNewNeighbor()
    {

	// @@@ TODO
	// picking a new neighbor depends on downloads (should that be
	// automatic?), filter similarity, or random.
	// or maybe they've asked for stuff in the past that you've had
	while (interestedPeers_.size() > 0) {
	    int test = (int)Math.random() * interestedPeers_.size();
	    PeerMsgOffer pmo = (PeerMsgOffer) interestedPeers_.remove(test);
	    if (isNeighbor(pmo.buyerID()) != null) {
		continue;
	    }

	    // this guy is not a neighbor, so connect
	    PeerMsgRequestConnect pmrc =
		new PeerMsgRequestConnect(peerID(), pmo.buyerID());
	    Boolean connect = (Boolean) network().rpc(pmrc);
	    if (!connect.booleanValue())
		continue;  // Request to connect denied.
	    return manager().peer(pmo.buyerID());
	}

	// Choose random peer to connect.
	Random rand = new Random();
	for (int i = 0; i < 10; i++) {
	    Peer neighbor = manager().randomPeer(rand);

	    if (neighbor.peerID() == peerID() ||
		isNeighbor(neighbor.peerID()) != null)
		continue;
	    else if (localTrust(neighbor.peerID()) <= BAD_PEER)
		continue;

	    PeerMsgRequestConnect pmrc =
		new PeerMsgRequestConnect(peerID(), neighbor.peerID());
	    Boolean connect = (Boolean) network().rpc(pmrc);
	    if (!connect.booleanValue())
		continue;  // Request to connect denied.

	    return neighbor;
	}
	return null;
    }


    protected Peer pickNeighborToRemove()
    {
	if (neighbors() == 0) return null;

	// @@@ for now, choose random
	//int x = (int) (Math.random() * neighbors());
	//return neighbor(x);

	// Removing peers is done by profitability
	// Current code done by profitability, with some probability
	Peer lt = null;

	double sumNeighborProfit = 0.0;
	ArrayList sortLT = new ArrayList();

	for (int i = 0; i < neighbors(); i++) {
	    int id = neighbor(i).peerID();

	    double profit = getProfit(id);
	    //if (profit > 0)
	    //System.out.println("huh?");
	    sumNeighborProfit += profit;
	    int pos = 0;
	    for ( ; pos < sortLT.size(); pos++) {
		int pid = ((Integer)sortLT.get(pos)).intValue();
		if (getProfit(pid) > profit)
		    break;
	    }
	    sortLT.add(pos, new Integer(id));

	    if (lt == null || profit < getProfit(lt.peerID()))
		lt = neighbor(i);
	}

	if (getProfit(lt.peerID()) <= 0.0)
	    return lt;

	double rand = Math.random();
	double[] dist = new double[sortLT.size()];
	for (int i = 0; i < sortLT.size(); i++)
	    dist[i] = getProfit(((Integer)sortLT.get(i)).intValue()) /
		sumNeighborProfit;

	int pid = 0;
	int i   = dist.length - 1;
	for (double sum = dist[i]; i > 0 && sum < rand; i--, sum += dist[i])
	    pid++;
	pid = ((Integer) sortLT.get(pid)).intValue();

	return manager().peer(pid);
    }

    // ==================== MESSAGE INTERFACE =============================

    /**
     * Dispatches message to associated message handler.
     * @param msg message received
     */
    public void message(PeerMsg msg)
    {
	if (msg.source() == peerID())
	    return;   // this is my message.

	// determine type of message and call corresponding message
	// handler.  Handle in order of specificity
	if (msg instanceof PeerMsgOffer)
	    handlePeerMsgOffer((PeerMsgOffer)msg);
	else if (msg instanceof PeerMsgOfferBuy)
	    handlePeerMsgOfferBuy((PeerMsgOfferBuy)msg);
	else if (msg instanceof PeerMsgRTR)
	    handlePeerMsgRTR((PeerMsgRTR)msg);
	else if (msg instanceof PeerMsgQuery)
	    handlePeerMsgQuery((PeerMsgQuery)msg);
	else if (msg instanceof PeerMsgQueryResponse)
	    handlePeerMsgQueryResponse((PeerMsgQueryResponse)msg);
	else if (msg instanceof PeerMsgTrust)
	    handlePeerMsgTrust((PeerMsgTrust)msg);
    }

  /**
   * Handles offer messages.
   * @param msg        offer message
   */
  protected void handlePeerMsgOffer(PeerMsgOffer msg)
  {
    if (msg.buyerID() == 0) {
      System.out.print("");
    }

    if (messages_.get(msg) != null) {
      ((PeerManagerRTR)manager()).numDuplicateRejects++;
      ((PeerManagerRTR)manager()).numDuplicateRejectsTTL[msg.ttl()+1]++;
      return;   // have already seen this message.
    }
    messages_.put(msg, msg);

    if (((PeerBehaviorRTR)behavior()).buyRTR(msg)) {
      // @@@ rather than do this, why not just call a function
      // on the other peer?  more efficient
      //PeerMsg msgr = new PeerMsgOfferBuy(msg);
      //network().send(msgr.destination(), msgr);

      // Are we using the RTR protocol?
      PeerRTR seller = (PeerRTR) manager().peer(msg.source());
      seller.handleBuyRTR(msg);

      // count sending the msg, w/o actually sending
      network().send(-1, msg);

      // take note that we paid this money to the neighbor
      getRevenueTracker(msg.source()).addSellRTRPayment(msg.price());

      // Record that we responded to this peer.  We may want to
      // connect to him later
      addInterestedPeer(msg);

      // if we buy an RTR, then we will also receive it later on
      handlePeerMsgRTR(msg);
    }
  }

    public void handlePeerMsgOfferBuy(PeerMsgOfferBuy msg) {
	balance_ += msg.price();

	// @@@ maybe do some bookkeeping on what
	// file/category it was for, etc

	// increase the buyer revenue of the neighbor who just bought from us
	getRevenueTracker(msg.source()).addBuyRTRRevenue(msg.price());

	// increase the seller revenue of the neighbor who sold this rtr
	// to us, if such a neighbor exists
	if (msg.buyerID() != peerID())
	    getRevenueTracker(msg.connection()).addSellRTRRevenue(msg.price());

	// Send the actual RTR
	PeerMsg msgr = new PeerMsgRTR(msg);
	network().send(msgr.destination(), msgr);
    }

    /**
     * Handles offer buy messages.
     * @param msg        offer buy message.  SOURCE is US, DEST is THEM
     */
    public void handleBuyRTR(PeerMsgOffer msg) {
	balance_ += msg.price();

	// @@@ maybe do some bookkeeping on what
	// file/category it was for, etc

	// increase the buyer revenue of the neighbor who just bought from us
	getRevenueTracker(msg.destination()/*source()*/)
	    .addBuyRTRRevenue(msg.price());

	// increase the seller revenue of the neighbor who sold this rtr
	// to us, if such a neighbor exists
	if (msg.buyerID() != peerID())
	    getRevenueTracker(msg.connection())
		.addSellRTRRevenue(msg.price());

	// Send the actual RTR
	//PeerMsg msgr = new PeerMsgRTR(msg);
	//network().send(msgr.destination(), msgr);

	// count sending the msg, w/o actually sending
	network().send(-1, msg);
    }

    /**
     * Handles rtr messages.
     * @param msg        rtr message
     */
    public void handlePeerMsgRTR(PeerMsgRTR msg) {
	queryResponseAttempts(queryResponseAttempts()+1);

	if (debugLevel() > 1)
	    System.out.println("Peer "+peerID()+" processes query.");

	if (behavior().respond(msg)) {
	    // @@@ TODO: change this to contain ALL approximate and exact
	    // matches
	    // @@@ notice that we respond to the buyer ID, and that the
	    // "connection" is the peer that sent me the rtr (the source)
	    // @@@ this may change depending on content model...
	    Content cont = content().findCategory(msg.category());
	    PeerMsg msgr = new PeerMsgRTRResponse(peerID(), msg.buyerID(),
						  msg.category(),
						  msg.file(),
						  msg.source(),
						  msg.ttl(),
						  cont);
	    network().send(msg.buyerID(), msgr);
	}

	// @@@ TODO in final simulation
	// don't forward it yet.  Instead, wait for the next "mini-step"
	// within the cycle.  Otherwise, peers that are 7 hops away
	// will sell more RTRs than peers that are very close
	newRTR_.addElement(msg);
    }

    protected Double handleRequestDownload(PeerMsgRequestDownload msg) {

	goodUploads(goodUploads()+1);

	// @@@ update file buyer's reputation, and the local profitability
	// score of the neighbor
	int peerID = msg.source(); // this is the buyer
	Peer downloader = manager().peer(peerID);
	Integer peerKey = new Integer(peerID);
	Double  popn    = (Double)localTrust_.get(peerKey);
	if (popn == null)
	    popn = new Double(0);

	// update buyer reputation
	double updateAmount = behavior().opinion(downloader, true);

	Double peerTrust = new Double(popn.doubleValue() + updateAmount);

	if (peerTrust.intValue() != 0)
	    localTrust_.put(peerKey, peerTrust);
	else localTrust_.remove(peerKey);

	// update buyer and connection profitability
	int connID = msg.connection(); // this is the peer who sold us the rtr
	RevenueTracker  connRt    = getRevenueTracker(connID);
	RevenueTracker  buyerRt  = getRevenueTracker(peerID);

	connRt.addSellRTRRevenue(PeerBehaviorRTR.FILE_PRICE);
	if (connID != peerID) // in case rtr seller = file buyer
	    buyerRt.addSellRTRRevenue(PeerBehaviorRTR.FILE_PRICE);

	// If we are using an adaptive topology, connect to
	// the peer who is downloading from you, b/c they are the "cash cow"
	if (manager().adaptiveTopology() != PeerManager.AT_NONE) {

	    // connect to the downloader, if we are not currently a
	    // neighbor
	    Peer neighbor;
	    if ((neighbor=isNeighbor(msg.source())) == null) {
		Peer ntr = pickNeighborToRemove();

		// Is there someone we can remove?
		if (ntr != null ) {
		    PeerMsg pmrc = new PeerMsgRequestConnect(peerID(),
							     peerID);

		    // did we successfully add the new neighbor?
		    if (((Boolean)network().rpc(pmrc)).booleanValue()) {
			manager().peer(peerID).addNeighbor(this);
			addNeighbor(manager().peer(peerID));

			// only disconnect if we successfully connected?
			network().rpc( new PeerMsgDisconnect(peerID(),
							     ntr.peerID()) );
			disconnects_++;

			// remove the revenue tracker
			removeRevenueTracker(ntr.peerID());
		    }
		}
	    }

	    // prune off neighbors, if I have too many
	    for (int i = 0; i < 5 && neighbors() >
		     NetworkManager.MAX_CONNECTIONS; i++) {
		neighbor = pickNeighborToRemove();
		if (neighbor != null) {
		    network().rpc( new PeerMsgDisconnect(peerID(),
							 neighbor.peerID()) );
		    disconnects_++;
		    removeRevenueTracker(neighbor.peerID());
		}
	    }
	}

	return new Double( Content.GOOD_FILE );
    }

    // A peer wants to connect to us.  Do we agree?
    protected Boolean handleRequestConnect(PeerMsgRequestConnect msg)
    {
	// if local trust is too low, forget about it
	if (localTrust(msg.source()) < 0)
	    return new Boolean(false);

	// If I have too many neighbors, can't connect
	else if (neighbors() > NetworkManager.MAX_CONNECTIONS)
	    return new Boolean(false);

	if (neighbors() < NetworkManager.MAX_CONNECTIONS)
	    return new Boolean(true);
	else if (neighbors() > 0) {
	    /* If i'm going to connect to a new peer I must remove a
	     * current neighbor.  Make sure that it is worth it.
	     * How can I tell it is worth it, if all local trusts are the
	     * same?  I guess if this guy is not profitable
	     */
	    Peer neighborToRemove = pickNeighborToRemove();
	    if (getProfit(neighborToRemove.peerID()) < 0)
		return new Boolean(true);
	}
	return new Boolean(false);
    }

//================================================================
// Peer Manager interface.

    /**
     * Initialize a new cycle.
     */
    public void newCycle() {
	super.newCycle();

	bHasQuery = false;
	newRTR_.clear();
	prevRTR_.clear();

	// update revenue trackers
	for (Enumeration keys = connectionOpinions_.keys();
	     keys.hasMoreElements(); ) {
	    RevenueTracker rt = (RevenueTracker)
		connectionOpinions_.get(keys.nextElement());
	    rt.newCycle();
	}

	if (exactMatches_.size() == cycle()) {
	    int prev = ((Integer) exactMatches_.lastElement()).intValue();
	    exactMatches_.addElement(new Integer(0));

	    double prevd = ((Double) totalScores_.lastElement()).doubleValue();
	    totalScores_.addElement(new Double(0));
	}

	if (cycle() == 0)  {
	    // @@@ do stuff
	}
	else {
	    // @@@ do other stuff
	}
    }

    // at the end of the cycle, everyone processes responses
    public void endCycle() {
	if (bHasQuery)
	    selectFileForDownload();

	if (cycle() % 5 == 0) {
	    if (manager().adaptiveTopology() != PeerManager.AT_NONE)
		changeNeighbors(); // switch things around
	}
    }

    // change your new RTRs to your prev RTRs
    public void initStep() {
	Vector temp = prevRTR_;
	prevRTR_ = newRTR_;
	newRTR_ = temp;
	newRTR_.clear();

	if (peerID() % 50 == 0)
	    System.out.println("num rtr " + prevRTR_.size());
    }

    public void step() {

	// forward messages from the previous step to all neighbors
	int numRTR = prevRTR_.size();
	for (int m = 0; m < numRTR; m++) {
	    PeerMsgRTR msg = (PeerMsgRTR) prevRTR_.elementAt(m);

	    for (int i = 0; i < neighbors(); i++) {
		Peer neigh = neighbor(i);
		if (((PeerBehaviorRTR)behavior()).sellRTR(msg, neigh)) {
		    double price =
			((PeerBehaviorRTR)behavior()).priceRTR(msg, neigh);
		    double probBuy = ((PeerBehaviorRTR)behavior())
			.probBuyRTR(msg, neigh);

		    PeerMsg msgr = new PeerMsgOffer(msg, neigh.peerID(), price);
		    network().send(neigh.peerID(), msgr);
		    ((PeerManagerRTR)manager()).numOffers++;
		    ((PeerManagerRTR)manager()).numOffersTTL[msgr.ttl()]++;

		    // take note that we expect something from this neighbor
		    getRevenueTracker(neigh.peerID())
			.addBuyRTRExpected(price * probBuy);
		}
	    }
	}
    }

// ===================================================================


  /**
   * Randomly issues query based on peer interest.
   * Sends out query message and puts peer into inactive state.
   * Peer will query for a file rank in some content category.
   * @param TTL broadcast search time-to-live
   */
  protected void issueQuery(int TTL)
  {
      bHasQuery = true;

      // @@@ this part may change if we change content model
      PeerContentManager pcm = manager_.contentManager();
      category_ = (int) (Math.random() * pcm.contentCategories()) + 1;
      file_     = (int) (Math.random() * pcm.uniqueFiles(category_));

      if (behavior().type() < PeerBehavior.MALICIOUS_PEER) {
	  // @@@ i thought peers had some special interest?
	  // ooh, randomCategory() is weighted by interest
	  category_ = content().randomCategory();
	  file_ = content().randomFile(category_);
      }

      // @@@
      // select a price for your neighbors.  Perhaps make it low? since we
      // want to maximize likelihood of getting the file we want, NOT the
      // likelihood necessarily of making money off TTLs. Interesting...
      // guard against bogus attacks this way? oh no, not if we are in a
      // malicious collective and our neighbors sell expensive rtrs for
      // us...
      // But maybe good peers will want to sell RTRs at a low price...since
      // getting what you want is the highest utility of all.

      // For now, make the price zero!

      // @@@ TODO ask ty: how do we get the "universally accepted" global
      // eigentrust value for a peer?  is it just globalTrust()?

      PeerMsgOffer msg = null;
      // forward message to all neighbor peers
      for (int i = 0; i < neighbors(); i++) {
	  Peer neigh = neighbor(i);

	  // I am the source, the connection, and the buyer
	  msg = new PeerMsgOffer(peerID(), neigh.peerID(), TTL,
				 category_, file_, peerID(),
				 0, globalTrust(), peerID());
	  network().send(neigh.peerID(), msg);
	  ((PeerManagerRTR)manager()).numOffers++;
	  ((PeerManagerRTR)manager()).numOffersTTL[msg.ttl()]++;
      }

      if (debugLevel() >= 1)
	  System.out.println("Peer "+peerID_+" issues query in category "
			     + category_ +" and file rank "+file_);
  }


//========================================================================
// Query responses.


    /**
     * Process responses to query.
     * @param cycle the cycle
     */
    protected void processQueryResponses()
    {
	// actually do nothing
	// This function is replaced by selectFileForDownload
    }

    protected void handleVoidDownload() {
	// no responses, then we are done.
	voidDownloads(voidDownloads()+1);

	// Change neighbors on void downloads.
	if (manager().adaptiveTopology() != PeerManager.AT_NONE &&
	    voidDownloads() % 3 == 0                            &&
	    behavior().type() < PeerBehavior.MALICIOUS_PEER)
	    changeNeighbors();

	return;
    }

    protected void selectFileForDownload() {
	int numResponses = queryResponses();
	if (numResponses == 0) {
	    handleVoidDownload();
	    return;
	}

	// @@@ TODO
	//  choose download according to exact match vs non-exact match
	int numIndivFiles = 0;
	for (int i=0; i < numResponses; i++) {
	    PeerMsgRTRResponse msgr =
		(PeerMsgRTRResponse)queryResponses_.elementAt(i);
	    numIndivFiles += msgr.cont().files();
	}

	Content category = content().findCategory(category_);
	double scores[] = new double[numIndivFiles];
	int message[] = new int [numIndivFiles];
	int file[] = new int[numIndivFiles];

	double scoreSum = 0;
	int numExact = 0;
	int count = 0;
	for (int i=0; i < numResponses; i++) {
	    PeerMsgRTRResponse msgr =
		(PeerMsgRTRResponse)queryResponses_.elementAt(i);

	    Content cont = msgr.cont();
	    int numFiles = cont.files();
	    for (int j=0; j < numFiles; j++) {
		int fileID = cont.file(j);
		message[count] = i;
		file[count] = fileID;

		if (category.containsFile(fileID))
		    scores[count] = 0; // i already have file
		else if (fileID == file_) {
		    scores[count] = PeerBehaviorRTR.EXACT_MATCH_SCORE;
		    numExact++;
		}
		else
		    scores[count] = PeerBehaviorRTR.APPROX_MATCH_SCORE;

		scoreSum += scores[count];
		if (count > 0)
		    scores[count] += scores[count-1]; // make it cumulative
		count++;
	    }
	}

	// perhaps all the files we received, we already have?
	if (scoreSum == 0) {
	    handleVoidDownload();
	    return;
	}

	// If this is not a void download, record the results
	exactMatches(exactMatches() + numResponses);//numExact);
	totalScore(totalScore() + scoreSum);

	// Using the CDF and a random number, select a file
	int choice = 0;
	double r = Math.random() * scoreSum;
	for (; choice < numIndivFiles; choice++) {
	    if (r < scores[choice])
		break;
	}

	// We have selected a file, now request the download from the
	// owner
	if (choice == numIndivFiles)
	    System.out.println("choice " + choice + " numindivfiles " +
			       numIndivFiles);
	PeerMsgQueryResponse msgr =
	    (PeerMsgQueryResponse)queryResponses_.elementAt(message[choice]);
	int remotePeerId = msgr.source();

	if (debugLevel() >= 1)
	    System.out.println("Peer " + peerID_ +
			       " requests non-trusted download from peer " +
			       remotePeerId);

	PeerMsg msg = new PeerMsgRequestDownload(peerID(), remotePeerId,
						 msgr.category(),
						 file[choice],
						 msgr.connection());

	double downloadStatus = ((Double)network().rpc(msg)).doubleValue();

	// update opinion matrix
	recordDownload(remotePeerId, msgr.connection(), downloadStatus);

	// count number of entirely failed downloads
	if (downloadStatus != Content.GOOD_FILE) {
	    failedDownloads(failedDownloads()+1);
	}
	else if (behavior().type() < PeerBehavior.MALICIOUS_PEER) {
	    // @@@ FOR NOW, to fix bug
	    /*
	    Content content = content().category(category_);
	    // Add category and file to content.
	    content.addFile(file[choice]);

	    manager().contentManager().addFileCategoryCount(category_,
							    file[choice]);
	    */
	}
    }

  /** Updates opinion on peer.
   * @param peerID ID of peer on which to update opinion
   * @param connection The connection used to send initial query.
   * @param download success of download
   */
  protected void recordDownload(int peerID, int connection, double download)
  {
      // @@@
      //  We don't really need to update reputations on download.  We need
      // to update them during *upload* instead.  Also, we may reconnect on
      // upload, rather than download

      // update statistics
      // download should always be good
      if (download == Content.GOOD_FILE) {
	  goodDownloads(goodDownloads()+1);
	  Peer    peer    = manager().peer(peerID);
 	  goodResponse( peer );
      }
  }

    //----------------------------
    // Functions for plots
    public int exactMatches()
    {
	Integer value = (Integer)exactMatches_.elementAt(cycle());
	return value.intValue();
    }

    public double totalScore()
    {
	Double value = (Double)totalScores_.elementAt(cycle());
	return value.doubleValue();
    }

    public void exactMatches(int num)
    {
	int index = exactMatches_.size() - 1;
	if (index < 0)
	    exactMatches_.addElement(new Integer(num));
	else
	    exactMatches_.setElementAt(new Integer(num), index);
    }

    public void totalScore(double num)
    {
	int index = totalScores_.size() - 1;
	if (index < 0)
	    totalScores_.addElement(new Double(num));
	else
	    totalScores_.setElementAt(new Double(num), index);
    }

    // Rank peers interest by how many approximate matches there are
    protected void addInterestedPeer(PeerMsgOffer msg)
    {
	int category = msg.category(); // or subcategory...
	Content content = content().findCategory(category);
	if (content == null)
	    return;
	int catSize = content.files();

	int len = interestedPeers_.size();
	int i;
	for (i=0; i < len; i++) {
	    PeerMsgOffer pmo = (PeerMsgOffer) interestedPeers_.elementAt(i);

	    int size = content().findCategory(pmo.category()).files();
	    if (size <= catSize) {
		interestedPeers_.insertElementAt(msg, i);
		break;
	    }
	}
	if (i == len && len < 10) {
	    interestedPeers_.addElement(msg);
	}
	if (len > 10) {
	    interestedPeers_.remove(len-1);
	}
    }
}
