/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.util;

import java.awt.*;
import java.awt.geom.*;


import qcsim.peer.*;

 /*
  * Query Cycle Simulator
  * P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /** 
  * Graphic Description of a download.
  * @author unascribed
  * @version 1.0
  */

public class DownloadDescription
{
  private Color color_;

  private Point2D pos_;

  public Peer start_;

  public Peer end_;

  private double slope_;

  private double offset_;

  public DownloadDescription(Peer start, Peer end, Color color)
  {
    color_  = color;
    start_  = start;
    end_    = end;
    pos_    = start.description().position();
    slope_  = (start.description().position().getY() - 
               end.description().position().getY()) / 
              (start.description().position().getX() - 
               end.description().position().getX());
    offset_ = start.description().position().getY() - 
              slope_ * start.description().position().getX();
  }

  public void drawPeers(Graphics g, boolean colorTrust)
  {
    synchronized (g)
    {
      start_.description().draw(g, start_.description().color(colorTrust));
      end_.description().draw(g, end_.description().color(colorTrust));
    }
  }

  public boolean draw(Graphics g, boolean animate)
  {
    boolean done = false;

    if (animate)
    {
      double  x    = pos_.getX();

      if (start_.description().position().getX() < 
          end_.description().position().getX())
      {
        if (x+1 > end_.description().position().getX()) 
          x = end_.description().position().getX();
        else x+=1;
        done = (x >= end_.description().position().getX());
      } else {
        if (x-1 < end_.description().position().getX()) 
          x = end_.description().position().getX();
        else x-=1;
        done = (x <= end_.description().position().getX());
      }

      synchronized (g)
      {
        pos_ = new Point2D.Double(x, slope_ * x + offset_);
        g.setColor(color_);
        g.drawRect((int)(pos_.getX() - 1), (int)(pos_.getY() - 1), 4, 4);
        g.fillRect((int)(pos_.getX() - 1), (int)(pos_.getY() - 1), 4, 4);
      }
    }
    else
    {
      synchronized (g)
      {
        g.setColor(color_);
        g.drawLine((int)start_.description().position().getX(), 
                   (int)start_.description().position().getY(),
                   (int)end_.description().position().getX(), 
                   (int)end_.description().position().getY());
        drawArrow(g);
        done = true;
      }
    }

    return done;
  }

  private void drawArrow(Graphics g)
  {
    // arrowhead sharpness
    double theta = Math.toRadians( 20 );
    // arrowhead length
    int size = 8;

    int x1 = (int) start_.description().position().getX();
    int y1 = (int) start_.description().position().getY();
    int x2 = (int) end_.description().position().getX();
    int y2 = (int) end_.description().position().getY();

    double angle;

    // calculate points for arrowhead
    angle=Math.atan2( y2-y1, x2-x1 )+Math.PI;

    int x3=(int)( x2+Math.cos( angle-theta )*size );
    int y3=(int)( y2+Math.sin( angle-theta )*size );

    int x4=(int)( x2+Math.cos( angle+theta )*size );
    int y4=(int)( y2+Math.sin( angle+theta )*size );

    // draw arrowhead
    g.drawLine( x2, y2, x3, y3 );
    g.drawLine( x2, y2, x4, y4 );
    g.drawLine( x3, y3, x4, y4 );
  }

}
