/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.util;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;

import qcui.util.*;

public class Plot
{
  public static final int BAR_GRAPH  = 1;
  public static final int LINE_GRAPH = 2;

  private String dataDir_;

  private Vector xpos_;

  private Vector ypos_;

  private double maxX_;

  private double maxY_;

  private String xlabel_;

  private String ylabel_;

  public Plot(String resource, String xlabel, String ylabel)
  {
    dataDir_ = resource + "/graphDat/";
    xpos_    = new Vector();
    ypos_    = new Vector();

    maxX_ = 0;
    maxY_ = 0;

    xlabel_ = xlabel;
    ylabel_ = ylabel;
  }

  public int plots()
  {
    return xpos_.size();
  }

  public double plotX(int i)
  {
    return ((Double)xpos_.elementAt(i)).doubleValue();
  }

  public double maxX()
  {
    return maxX_;
  }

  public void maxX(double x)
  {
    maxX_ = x;
  }

  public String xlabel()
  {
    return xlabel_;
  }

  public double plotY(int i)
  {
    return ((Double)ypos_.elementAt(i)).doubleValue();
  }

  public double maxY()
  {
    return maxY_;
  }

  public void maxY(double y)
  {
    maxY_ = y;
  }

  public String ylabel()
  {
    return ylabel_;
  }

  public void plot(double x, double y)
  {
    maxX_ = (maxX_ < x) ? x : maxX_;
    maxY_ = (maxY_ < y) ? y : maxY_;

    for (int i = 0; i < xpos_.size(); i++)
    {
      if (plotX(i) == x)
      {
        xpos_.setElementAt(new Double(x), i);
        ypos_.setElementAt(new Double(y), i);
        return;
      }
      else if (x < plotX(i))
      {
        xpos_.insertElementAt(new Double(x), i);
        ypos_.insertElementAt(new Double(y), i);
        return;
      }
    }
    xpos_.addElement(new Double(x));
    ypos_.addElement(new Double(y));
  }

  public void clear()
  {
    xpos_.clear();
    ypos_.clear();
    maxX_ = 0;
    maxY_ = 0;
  }

  public void flush(String filename)
  {
    try {
      FileOutputStream file = new FileOutputStream(dataDir_ + filename);

      for (int i = 0; i < plots(); i++)
      {
        String p = plotX(i) + " " + plotY(i) + "\n";
        file.write( p.getBytes() ); 
      }
    } catch (Exception e) {
      System.err.println( e );
    }
  }

  public void graph(Graphics g, JComponent view, int type, 
                    double scaleX, double scaleY, 
                    int graphX, int graphY, Dimension dim)
  {
    int width  = (int) dim.getWidth();
    int height = (int) dim.getHeight();

    g.setColor(Color.black);
    g.drawLine(graphX, graphY, graphX+width, graphY);   // Draw x axis
    g.drawLine(graphX, graphY, graphX, graphY-height);  // Draw y axis
    g.drawString(xlabel(), graphX + (int)(width * 0.25), graphY+30);

    VTextIcon ylabel = new VTextIcon(new JPanel(), ylabel(),
                                     VTextIcon.ROTATE_LEFT);
    ylabel.paintIcon(view, g, graphX-60, graphY-height);

    for (double i = 1.0; i <= scaleX; i++)
    {
      int xpos = graphX + (int) ((i*width) / scaleX);

      double valx = (i * maxX()) / scaleX;

      if (valx > scaleX)
        g.drawString(Integer.toString((int)valx), xpos, graphY+15);
      else
        g.drawString(Double.toString(valx), xpos, graphY+15);
    }

    for (double i = 1.0; i <= scaleY; i++)
    {
      int ypos = graphY - (int) ((i*height) / scaleY); 

      double valy = (i * maxY()) / scaleY; 

      valy = Math.floor(valy * 100.0);
      valy /= 100.0;

      if (valy > scaleY)
        g.drawString(Integer.toString((int)valy), graphX-35, ypos);
      else
        g.drawString(Double.toString(valy), graphX-35, ypos);
    }

    if (type == BAR_GRAPH)
    {
      for (int i = 0; i < plots(); i++)
      {
        int x1 = graphX + (int) ((plotX(i)*width)  / maxX());
        int y1 = graphY - (int) ((plotY(i)*height) / maxY());
  
        g.fillRect(x1, y1, 2, graphY - y1);
      } 
    }
    else if (type == LINE_GRAPH)
    {
      int x1=0, x2=0;
      int y1=0, y2=0;

      for (int i = 0; i < plots(); i++)
      {
        x1 = (int) ((plotX(i) * width) / maxX());
        x1 = graphX + x1;
        y1 = graphY - (int)((plotY(i) * height) / maxY());
  
        if (i > 0) g.drawLine(x1, y1, x2, y2);
  
        x2 = x1;
        y2 = y1;
      }
    }
  }

}
