/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcsim.util;

import java.io.*;
import java.util.*;

import qcsim.*;
import qcsim.impl.*;
import qcsim.peer.*;
import qcsim.peer.impl.PeerBehaviorRTR;
import qcsim.peer.impl.PeerManagerRTR;


public class QueryCycleScriptParser
{
  private boolean adaptiveTopology_;

  private int cycles_;

  private File script_;

  private String resource_;

  public QueryCycleScriptParser(String resource)
  {
    resource_ = resource;

    System.out.print("Enter Script File Name: ");
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));

    try {
       String script = br.readLine();
       script_ = new File(resource + "/scripts/" + script.trim());
       if (!script_.exists())
       {
         System.out.println("File does not exist!  " + script_.getPath());
         System.exit(1);
       }
    } catch (Exception ioe) {
       System.out.println("IO error trying to read script name! " + ioe);
       System.exit(1);
    }

    System.out.print("How many cycles? ");
    br = new BufferedReader(new InputStreamReader(System.in));

    try {
       cycles_ = Integer.parseInt(br.readLine().trim());
    } catch (Exception ioe) {
       System.out.println("Error trying to read cycles!  " + ioe);
       System.exit(1);
    }

    adaptiveTopology_ = true;
    /*
    System.out.print("Adaptive Topology (yes/no)? ");
    br = new BufferedReader(new InputStreamReader(System.in));

    try {
       String at = br.readLine().trim();
       adaptiveTopology_ = at.startsWith("yes");
       if (adaptiveTopology_) System.err.println("ADAPTIVE TOPOLOGY OK");
    } catch (Exception ioe) {
       System.out.println("Error trying to adaptive topology (yes or no)!  " + ioe);
       System.exit(1);
    }
    */

  }

  public int cycles() { return cycles_; }

  public QueryCycleManager parse()
  {
    QueryCycleManager manager = new QueryCycleManagerImpl(resource_);

    processScript(manager);

    if (adaptiveTopology_)
      manager.peerManager().adaptiveTopology(PeerManager.AT_ALL);
    return manager;
  }

  private void processScript(QueryCycleManager manager)
  {
    BufferedReader in = null;

    try {
      in = new BufferedReader(new FileReader(script_));
    } catch (IOException e) {
      System.out.println("File "+script_.getPath()+" was not found.");
      return;
    }

    // read in file
    while (true)
    {
      Vector command      = null;
      String inputCommand = "";
      try {
        inputCommand = in.readLine();
        if (inputCommand == null) return;
        command = parseCommand(inputCommand);
        if (command.elementAt(0).equals("filterLevel"))      
        {
          command.addElement(parseCommand(in.readLine()));
          command.addElement(parseCommand(in.readLine()));
        }
        else if (command.elementAt(0).equals("fudgeFactor"))
        {
          command.addElement(parseCommand(in.readLine()));
          command.addElement(parseCommand(in.readLine()));
        }
      } catch (IOException e) {}

      processCommand(manager, command);
    }
  }

  public static Vector parseCommand(String command)
  {
    Vector commandVector = new Vector();
    int oldi = 0;
    int i    = 0;

    for ( ;i < command.length() && command.charAt(i) != '\n'; oldi = i)
    {
      for ( ;i < command.length() && command.charAt(i) == ' '; i++)
        ;
      if (command.charAt(i) == '\n')
        break;
      for ( ;i < command.length() && command.charAt(i) != ' '; i++)
        ;
      commandVector.addElement(command.substring(oldi, i));
    }
    return commandVector;
  }

  private void processCommand(QueryCycleManager manager, Vector inputCommand)
  {
    if (inputCommand.size() == 0) return;

    String command = ((String)inputCommand.elementAt(0)).trim();

    if (command.startsWith(";;")) return;  // Ignore comments

    if (command.equals("cycles"))
    {
      if (inputCommand.size() != 7)
      {
        System.out.println("Wrong syntax: cycles NumberOfCyclesToSimulate");
        return;
      }
      cycles_ = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());

    }
    else if (command.equals("peers"))
    {
      if (inputCommand.size() != 7)
      {
        System.out.println("Wrong syntax: peers goodPeers goodPeerNeighbors maliciousPeers maliciousPeerNeighbors highlyTrustPeers highlyTrustedPeerNeighbors");
        return;
      }

      int gp   = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      int gpn  = Integer.parseInt(((String)inputCommand.elementAt(2)).trim());
      int mp   = Integer.parseInt(((String)inputCommand.elementAt(3)).trim());
      int mpn  = Integer.parseInt(((String)inputCommand.elementAt(4)).trim());
      int htp  = Integer.parseInt(((String)inputCommand.elementAt(5)).trim());
      int htpn = Integer.parseInt(((String)inputCommand.elementAt(6)).trim());


      manager.peers(gp, gpn, mp, mpn, htp, htpn);

      System.out.println("Peer network created.");
    }
    else if (command.equals("rtrPeers"))
    {
      int peers      = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      int neighbors  = Integer.parseInt(((String)inputCommand.elementAt(2)).trim());

      manager.rtrPeers(peers, neighbors);
    }
    else if (command.equals("adaptive"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();
      rtrManager.adaptiveTopology(1);
    }
    else if (command.equals("evolution"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();
      rtrManager.evolution(true);
    }
    else if (command.equals("profitType"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();
      int pType = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      rtrManager.profitType(pType);
    }
    else if (command.equals("rtrType"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();
      int rtrType = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());

      rtrManager.rtrType(rtrType);
    }
    else if (command.equals("filterLevel"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();
      Vector filters    = (Vector) inputCommand.elementAt(1);
      Vector dist       = (Vector) inputCommand.elementAt(2);   

      rtrManager.filterLevel(filters, dist);
    }
    else if (command.equals("fudgeFactor"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();
      Vector fudgeFactors       = null;
      Vector dist               = null;
      int    arrayLength        = 1;     // Default value 1
      
      if (inputCommand.elementAt(1) instanceof String)
      {
        arrayLength  = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
        fudgeFactors = (Vector)inputCommand.elementAt(2);
        dist         = (Vector)inputCommand.elementAt(3);
      }
      else
      {
        fudgeFactors = (Vector)inputCommand.elementAt(1);
        dist         = (Vector)inputCommand.elementAt(2);
      }

      rtrManager.fudgeFactor(arrayLength, fudgeFactors, dist);

    }
    else if (command.equals("probBuy"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();

      inputCommand.removeElementAt(0);
      rtrManager.probBuy(inputCommand);      
    }
    else if (command.equals("fracReturn"))
    {
      PeerManagerRTR rtrManager = (PeerManagerRTR) manager.peerManager();

      double fr = Double.parseDouble(((String)inputCommand.elementAt(1)).trim());
      rtrManager.fracReturn(fr);      
    }
    else if (command.equals("content"))
    {
      if (inputCommand.size() != 4)
      {
        System.out.println("Wrong syntax: content categories minCategories distribution");
        return;
      }
      int cat  = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      int min  = Integer.parseInt(((String)inputCommand.elementAt(2)).trim());
      int dist = Integer.parseInt(((String)inputCommand.elementAt(3)).trim());

      manager.content(cat, min, dist);
      System.out.println("Content created.");
    }
    else if (command.equals("maliciousPeerCategoryPref"))
    {
      if (inputCommand.size() != 2)
      {
        System.out.println("Wrong syntax: maliciousPeerCategoryPref pref");
        return;
      }
      int pref = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      manager.maliciousPeerCategoryPref(pref);
      System.out.println("Malicious Peer Category Pref Set.");

    }
    else if (command.equals("highTrustPeerCategoryPref"))
    {
      if (inputCommand.size() != 2)
      {
        System.out.println("Wrong syntax: highTrustPeerCategoryPref pref");
        return;
      }
      int pref = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      manager.highTrustPeerCategoryPref(pref);
      System.out.println("High Trust Peer Category Pref Set.");

    }
    else if (command.equals("goodPeerResponseSelector"))
    {
      if (inputCommand.size() != 2)
      {
        System.out.println("Wrong syntax: goodPeerResponseSelector selector");
        return;
      }

      int sel = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      manager.goodPeerResponseSelector(sel);
      System.out.println("Good Peer Response Selector Set.");
    }
    else if (command.equals("goodPeerTTL"))
    {
      if (inputCommand.size() != 2)
      {
        System.out.println("Wrong syntax: goodPeerTTL ttl");
        return;
      }

      int ttl = Integer.parseInt(((String)inputCommand.elementAt(1)).trim());
      manager.goodPeerTTL(ttl);
      System.out.println("Good Peer TTL Set.");

    }
    else if (command.equals("freeriderFraction"))
    {
      if (inputCommand.size() != 2)
      {
        System.out.println("Wrong syntax: freeriderFraction fraction");
        return;
      }

      double frac = Double.parseDouble(((String)inputCommand.elementAt(1)).trim());
      manager.freeriderFraction(frac);
      System.out.println("free-riders set.");
    }
    else if (command.equals("errorFraction"))
    {
      if (inputCommand.size() != 3)
      {
        System.out.println("Wrong syntax: errorFraction error type");
        return;
      }

      double frac = Double.parseDouble(((String)inputCommand.elementAt(1)).trim());
      int type = Integer.parseInt(((String)inputCommand.elementAt(2)).trim());

      manager.errorFraction(frac, type);
      System.out.println("errorFraction Set.");

    }
    else if (command.equals("setPeerActivity"))
    {
      if (inputCommand.size() < 3)
      {
        System.out.println("Wrong syntax: setPeerActivity peerID_1 [peerID_2 ...] activity");
        return;
      }

      int act = Integer.parseInt(((String)inputCommand.lastElement()).trim());
      int peerID = 0;

      Vector peers = new Vector();
      for (int i = 1; i < inputCommand.size() - 1; i++)
      {
        peerID = Integer.parseInt(((String)inputCommand.elementAt(i)).trim());
        peers.addElement(new Integer( peerID ));
      }

      manager.setPeerActivity(peers, act);
      System.out.println("Peer Activity Set.");
    }
    else if (command.equals("establishCollective"))
    {
      if (inputCommand.size() < 4)
      {
        System.out.println("Wrong syntax: establishCollective peerID_1 [peerID_2 ...] CollectiveType ConfigurationType");
        return;
      }

      int argc = inputCommand.size();
      int t1   = Integer.parseInt(((String)inputCommand.elementAt(argc-2)).trim());
      int t2   = Integer.parseInt(((String)inputCommand.elementAt(argc-1)).trim());

      int    peerID = 0;
      Vector peers  = new Vector();
      for (int i = 1; i < argc - 2; i++)
      {
        peerID = Integer.parseInt(((String)inputCommand.elementAt(i)).trim());
        peers.addElement(new Integer( peerID ));
      }

      manager.establishCollective(peers, t1, t2);
      System.out.println("Collective Set.");
    }
  }
}




