/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.font.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import java.net.*;


import qcui.view.*;
import qcsim.*;
import qcsim.util.*;
import qcsim.net.util.*;
import qcsim.peer.*;
import qcsim.peer.content.*;
import qcsim.peer.impl.PeerManagerRTR;

 /*
  * Title: Simulator Manager Implementation
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Create and initialize simulation.
  * @author unascribed
  * @version 1.0
  */

public class InitializationPanel extends JPanel
{
  public static final String LOAD_SIMULATOR = "Load Simulator";
  public static final String PEERS          = "Peers";
  public static final String CONTENT        = "Content";
  public static final String G_BEHAVIOR     = "Good Peer Behavior";
  public static final String H_BEHAVIOR     = "Highly Trusted Peer Behavior";
  public static final String M_BEHAVIOR     = "Malicious Peer Behavior";
  public static final String ACTIVITY       = "Peer Activity";
  public static final String COLLECTIVE     = "Collectives";
  public static final String RTR            = "RTR";

  private static final String COMMIT  = "Commit";
  private static final String OK      = "Ok";
  private static final String SCRIPT  = "Script";
  private static final String EXIT    = "Exit";


  private static final String DISPLAY_CONTROL_PANEL = "Display Control Panel";
  private static final String LOAD_PANEL       = "Load Simulator";
  private static final String PEERS_PANEL      = "Peers";
  private static final String CONTENT_PANEL    = "Content";
  private static final String G_BEHAVIOR_PANEL = "Good Peer Behavior";
  private static final String H_BEHAVIOR_PANEL = "Highly Trusted Peer Behavior";
  private static final String M_BEHAVIOR_PANEL = "Malicious Peer Behavior";
  private static final String ACTIVITY_PANEL   = "Peer Activity";
  private static final String COLLECTIVE_PANEL = "Collectives";

  // Hash values for number of peer fields.
  public static final String NUM_GP   = "num good peers";
  public static final String NUM_GPN  = "num good peer neighbors";
  public static final String NUM_HTP  = "num high trust peers";
  public static final String NUM_HTPN = "num high trust peer neighbors";
  public static final String NUM_MP   = "num malicious peers";
  public static final String NUM_MPN  = "num malicious peer neighbors";

  // Hash values for content fields
  public static final String DISTRIBUTION   = "Distribution";
  public static final String CATEGORIES     = "Categories";
  public static final String MIN_CATEGORIES = "min Categories";

  // Hash values for peer behavior
  public static final String HTP_CAT_PREF = "htp category preference";
  public static final String HTP_CAT_FRAC = "htp category fraction";
  public static final String HTP_ERR_FRAC = "htp error fraction";
  public static final String MP_CAT_PREF  = "mp category preference";
  public static final String MP_CAT_FRAC  = "mp category fraction";
  public static final String MP_ERR_FRAC  = "mp error fraction";
  public static final String GP_RESPONSE  = "gp response selection";
  public static final String GP_ERR_FRAC  = "gp error fraction";
  public static final String GP_TTL       = "gp ttl";
  public static final String GP_FREERIDER_FRAC = "Good Peer Freerider fraction";

  // Hash values for malicious collectives
  public static final String COLLECTIVE_TYPE  = "Collective type";
  public static final String COLLECTIVE_TRUST = "Collective trust";

  public static final String RTR_PEERS         = "rtr peers";
  public static final String RTR_NEIGHBORS     = "rtr neighbors";
  public static final String RTR_TYPE          = "rtr simulation type";
  public static final String RTR_FILTER        = "rtr filter";
  public static final String RTR_FILTER_RATIO  = "rtr filter ratio";
  public static final String RTR_FUDGE         = "rtr fudge";
  public static final String RTR_FUDGE_LIST    = "rtr fudge list";
  public static final String RTR_FUDGE_RATIO   = "rtr fudge ratio";
  public static final String RTR_PROBBUY       = "rtr prob buy";
  public static final String RTR_FRACRETURN    = "rtr fracreturn";
  public static final String RTR_BARGAINFACTOR = "rtr bargain factor";
  public static final String RTR_PROFIT        = "rtr profit";
  public static final String RTR_ADAPTIVE      = "rtr ADAPTIVE";
  public static final String RTR_EVOLUTION     = "rtr evolution";

  private String currentPanel_;

  private QueryCyclePanel    parent_;
  private InitializationView view_;

  private JPanel displayPanel_;
  private JPanel displayControlPanel_;
  private JPanel attributePanel_;

  private Hashtable savedSims_;

  private Hashtable swingComponents_;

  private Vector collectives_;

  /**
   * Constructor
   */
  public InitializationPanel(QueryCyclePanel parent)
  {
    super();
    parent_   = parent;
    swingComponents_ = new Hashtable();
    collectives_     = new Vector();

    setPreferredSize(parent.dimension());
    setMinimumSize(parent.dimension());
    setMaximumSize(parent.dimension());
    setLayout(new BorderLayout()); 

    initializePanel();
    
    displayPanel_ = new JPanel();
    add(displayPanel_, BorderLayout.CENTER);

    validate();
    repaint();
  }

  private void initializePanel()
  {
    JPanel controlPanel           = new JPanel();
    JPanel attributePanelControls = new JPanel();
    JPanel attributePanelCommands = new JPanel();

    attributePanelControls.setPreferredSize(new Dimension(200, 250));
    attributePanelControls.setMinimumSize(new Dimension(200, 250));
    attributePanelControls.setMaximumSize(new Dimension(200, 250));

    attributePanelCommands.setPreferredSize(new Dimension(200, 100));
    attributePanelCommands.setMinimumSize(new Dimension(200, 100));
    attributePanelCommands.setMaximumSize(new Dimension(200, 100));

    attributePanel_ = new JPanel();
    attributePanel_.setBorder(BorderFactory.createEtchedBorder());
    attributePanel_.setLayout(new CardLayout());
    attributePanel_.setPreferredSize(new Dimension(200, 300));
    attributePanel_.setMinimumSize(new Dimension(200, 300));
    attributePanel_.setMaximumSize(new Dimension(200, 300));

    displayControlPanel_ = new JPanel();
    displayControlPanel_.setPreferredSize(new Dimension(200, 100));
    displayControlPanel_.setMinimumSize(new Dimension(200, 100));
    displayControlPanel_.setMaximumSize(new Dimension(200, 100));
    attributePanel_.add(DISPLAY_CONTROL_PANEL, displayControlPanel_);

    controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
    controlPanel.add(attributePanelControls);
    controlPanel.add(attributePanel_);
    controlPanel.add(attributePanelCommands);

    // Now setup control and command attribute panels.

    // The attribute panel control action listener.
    ActionListener listener = new ActionListener() {
      public void actionPerformed(ActionEvent event) 
      {
        String command = event.getActionCommand();
        if (command.equals(COMMIT))
        {
          JTextField gpf   = (JTextField) swingComponents_.get(NUM_GP);
          int        peers = Integer.parseInt(gpf.getText());

          if (peers != 0) 
          {
            initHTPBehavior(null);
            initMPBehavior(null);
            initGPBehavior(null);
          }

          if (parent_.manager().contentManager() == null)
            initContent(null);

          parent_.initializationComplete();
        }
        else if (command.equals(OK))
        {
          CardLayout cards = (CardLayout) attributePanel_.getLayout();
          initialize( currentPanel_ );
          currentPanel_ = DISPLAY_CONTROL_PANEL;
          cards.show(attributePanel_, currentPanel_);
        }
        else if (command.equals(SCRIPT))
        {
          script();
        }
        else if (command.equals(EXIT))
        {
          System.exit(0);
        }
        else
        {
          CardLayout cards = (CardLayout) attributePanel_.getLayout();

          currentPanel_ = event.getActionCommand();
          cards.show(attributePanel_, currentPanel_);
        }
      }
    };

    ButtonGroup group = new ButtonGroup();
    attributePanel_.add(LOAD_PANEL, load(listener, group));

    if (parent_.mode() != QueryCyclePanel.MODE_RTR)  
      attributePanel_.add(PEERS_PANEL, peers(listener, group));

    attributePanel_.add(CONTENT_PANEL, content(listener, group));
    attributePanel_.add(G_BEHAVIOR_PANEL, goodBehavior(listener, group));
    attributePanel_.add(H_BEHAVIOR_PANEL, highTrustBehavior(listener, group));
    attributePanel_.add(M_BEHAVIOR_PANEL, maliciousBehavior(listener, group));
    attributePanel_.add(ACTIVITY_PANEL, activity(listener, group));
    attributePanel_.add(COLLECTIVE_PANEL, collective(listener, group));

    if (parent_.mode() == QueryCyclePanel.MODE_RTR ||  
        parent_.mode() == QueryCyclePanel.MODE_ALL) 
      attributePanel_.add(RTR, rtr(listener, group));

    // Setup attribute controls.

    attributePanelControls.setLayout(new GridLayout(9,1));
    attributePanelControls.setBorder(BorderFactory.createTitledBorder("Attribute Panel Controls"));

    for (Enumeration enum = group.getElements(); enum.hasMoreElements(); )
    {
      JToggleButton button = (JToggleButton) enum.nextElement();
      attributePanelControls.add(button);
    }

    // Setup attribute commands.
    attributePanelCommands.setLayout(new GridLayout(3,2));
    attributePanelCommands.setBorder(BorderFactory.createTitledBorder("Attribute Panel Commands"));

    JButton ok = new JButton(OK);
    ok.addActionListener(listener);
    attributePanelCommands.add(ok);

    JButton commit = new JButton(COMMIT);
    commit.addActionListener(listener);
    attributePanelCommands.add(commit);

    JButton script = new JButton(SCRIPT);
    script.addActionListener(listener);
    attributePanelCommands.add(script);

    JButton exit = new JButton(EXIT);
    exit.addActionListener(listener);
    attributePanelCommands.add(exit);

    add(controlPanel, BorderLayout.WEST);
  }

  private void initialize(String panel)
  {
    if (panel == null)
      return;
    else if (panel.equals(PEERS_PANEL))
      initPeers(null);
    else if (panel.equals(ACTIVITY_PANEL))
      initActivity(null);
    else if (panel.equals(COLLECTIVE_PANEL))
      initCollective(null);
    else if (panel.equals(LOAD_PANEL))
      loadSimulation();
    else if (panel.equals(CONTENT_PANEL))
      initContent(null);
    else if (panel.equals(G_BEHAVIOR_PANEL))
      initGPBehavior(null);
    else if (panel.equals(RTR))
      initRTR(null);
    repaint();
  }

  private void script()
  {
     final JDialog dialog = new JDialog((JFrame)parent_.container(), "Save");

     final JTextField filename = new JTextField();

     JPanel main = new JPanel();
     main.setLayout(new GridLayout(2,2));
     main.add(new JLabel("Script File Name"));
     main.add(filename);

     ActionListener listener = new ActionListener() {
       public void actionPerformed(ActionEvent event) 
       {
         String command = event.getActionCommand();
         if (command.equals("Save"))
         {
           String fname = parent_.scriptsDir()+"/"+filename.getText();

           try
           {
             FileOutputStream out = new FileOutputStream( fname );

             JTextField gpf   = (JTextField) swingComponents_.get(NUM_GP);
             int        peers = Integer.parseInt(gpf.getText());

             if (peers == 0) 
               initRTR(out);
             else
             {
               initPeers(out);
               initHTPBehavior(out);
               initMPBehavior(out);
               initGPBehavior(out);
               initCollective(out);
             }
             initContent(out);
             initActivity(out);

             out.flush();
             out.close();

           } catch (SecurityException e) {
             System.err.println("Security exception: " + e.getMessage());
           } catch (FileNotFoundException e) {
             System.err.println("File Not Found exception: " + e.getMessage());
           } catch (NotSerializableException e) {
             System.err.println("Not serializable exception: " + e.getMessage());
           } catch (Exception e) {
             e.printStackTrace();
             System.err.println("Simulation save exception: " + e.getMessage());
           }
         }
         dialog.dispose();
       }
     };

     JPanel  controls = new JPanel();
     JButton save     = new JButton("Save");
     JButton cancel   = new JButton("Cancel");
     controls.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 1));
     controls.add(save);
     controls.add(cancel);

     save.addActionListener(listener);
     cancel.addActionListener(listener);

     dialog.getContentPane().setLayout(new BorderLayout(30, 30));
     dialog.getContentPane().add(main, BorderLayout.CENTER);
     dialog.getContentPane().add(controls, BorderLayout.SOUTH);

     dialog.pack();
     dialog.setVisible(true);
     dialog.toFront();
  } 


//=======================================================

  private JPanel load(ActionListener listener, ButtonGroup group)
  {
    JToggleButton load  = new JToggleButton(LOAD_SIMULATOR);
    Hashtable     files = new Hashtable();
    URL           url   = null;
    JComboBox     combo = null;

    savedSims_ = new Hashtable();

    try {
      if (parent_.container() instanceof JApplet)
      {
        JApplet applet = (JApplet) parent_.container();
        url = new URL(applet.getCodeBase(), parent_.simulationsDir()+"/sims"); 
      }
      else
      {
        File file = new File(parent_.simulationsDir()+"/sims");
        url = file.toURL();
      }

      InputStream in=url.openStream(); 
      BufferedReader dir = new BufferedReader(new InputStreamReader(in)); 
      String file = null;
      Vector v    = new Vector();
  
      while ((file = dir.readLine()) != null)
      { 
        String name  = file.substring(0, file.indexOf(":"));
        String fname = file.substring(file.indexOf(":")+1, file.length());
        savedSims_.put(name, fname);
        v.addElement(name); 
      }

      combo = new JComboBox( v );
      in.close();
      dir.close(); 
    } catch (Exception e) {
      System.err.println("LOAD FILE SETUP: " + e);
      e.printStackTrace();
    }

    JPanel  panel = new JPanel();
    panel.setPreferredSize(new Dimension(200, 100));
    panel.setMinimumSize(new Dimension(200, 100));
    panel.setMaximumSize(new Dimension(200, 100));
    panel.setBorder(BorderFactory.createTitledBorder("Load Simulation"));

    panel.add( combo );
    swingComponents_.put(LOAD_SIMULATOR, combo);

    load.addActionListener(listener);
    group.add(load);

    return panel;
  }

  /**
   * Initialize display to setup power law network
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel peers(ActionListener listener, ButtonGroup group)
  {
    JToggleButton peers = new JToggleButton(PEERS);
    JPanel        panel = new JPanel();
    JPanel        p1    = new JPanel();
    JPanel        p2    = new JPanel();
    JPanel        p3    = new JPanel();
    JTextField    gpf   = new JTextField("100");
    JTextField    htpf  = new JTextField("5");
    JTextField    mpf   = new JTextField("10");
    JTextField    gpnf  = new JTextField("3");
    JTextField    htpnf = new JTextField("5");
    JTextField    mpnf  = new JTextField("5");

    swingComponents_.put(NUM_GP, gpf);
    swingComponents_.put(NUM_HTP, htpf);
    swingComponents_.put(NUM_MP, mpf);
    swingComponents_.put(NUM_GPN, gpnf);
    swingComponents_.put(NUM_HTPN, htpnf);
    swingComponents_.put(NUM_MPN, mpnf);

    panel.setLayout(new GridLayout(6, 1, 10, 10));
    panel.setPreferredSize(new Dimension(100, 100));
    panel.setMaximumSize(new Dimension(100, 100));

    p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
    p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
    p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));
    p1.setBorder(BorderFactory.createEtchedBorder(Color.black, Color.black));
    p2.setBorder(BorderFactory.createEtchedBorder(Color.blue, Color.blue));
    p3.setBorder(BorderFactory.createEtchedBorder(Color.red, Color.red));

    panel.add(p1);
    panel.add(p3);
    panel.add(p2);

    p1.add(new JLabel("Good      "));
    p1.add(gpf);
    p1.add(new JLabel("Neighbors "));
    p1.add(gpnf);

    p2.add(new JLabel("High Trust"));
    p2.add(htpf);
    p2.add(new JLabel("Neighbors "));
    p2.add(htpnf);

    p3.add(new JLabel("Malicious "));
    p3.add(mpf);
    p3.add(new JLabel("Neighbors "));
    p3.add(mpnf);

    peers.addActionListener(listener);
    group.add(peers);

    panel.setBorder(BorderFactory.createTitledBorder("Peer Network"));

    return panel;
  }

  private JPanel rtr(ActionListener listener, ButtonGroup group)
  {
    JToggleButton rtr           = new JToggleButton(RTR);
    JPanel        rtrPanel      = new JPanel();
    JTextField    peers         = new JTextField("100");
    JTextField    neighbors     = new JTextField("5");
    JComboBox     type          = new JComboBox();
    JComboBox     profit        = new JComboBox();
    JCheckBox     adaptive      = new JCheckBox("Adaptive");
    JCheckBox     evolution     = new JCheckBox("Evolutionary");
    JTextField    filter        = new JTextField("0 1 2");
    JTextField    filterRatio   = new JTextField("14 5 1");
    JTextField    fudge         = new JTextField("3");
    JTextField    fudgeList     = new JTextField(".2 .6 .8 .2 .6 .7");
    JTextField    fudgeRatio    = new JTextField("4 6");
    JTextField    probBuy       = new JTextField(".1 .2 .3");
    JTextField    fracReturn    = new JTextField(".5");
    JTextField    bargainFactor = new JTextField("1");

    swingComponents_.put(RTR, rtrPanel);
    swingComponents_.put(RTR_PEERS, peers);
    swingComponents_.put(RTR_NEIGHBORS, neighbors);
    swingComponents_.put(RTR_TYPE, type);
    swingComponents_.put(RTR_PROFIT, profit);
    swingComponents_.put(RTR_ADAPTIVE, adaptive);
    swingComponents_.put(RTR_EVOLUTION, evolution);
    swingComponents_.put(RTR_FILTER, filter);
    swingComponents_.put(RTR_FILTER_RATIO, filterRatio);
    swingComponents_.put(RTR_FUDGE, fudge);
    swingComponents_.put(RTR_FUDGE_LIST, fudgeList);
    swingComponents_.put(RTR_FUDGE_RATIO, fudgeRatio);
    swingComponents_.put(RTR_PROBBUY, probBuy);
    swingComponents_.put(RTR_FRACRETURN, fracReturn);
    swingComponents_.put(RTR_BARGAINFACTOR, bargainFactor);

    rtrPanel.setLayout(new  BoxLayout(rtrPanel, BoxLayout.Y_AXIS));

    type.addItem("Good Peer");
    type.addItem("Competitive Forward");
    type.addItem("All Forward");
    type.addItem("No Forward");
    rtrPanel.add(type);

    profit.addItem("Profit Seller");
    profit.addItem("Profit Buyer");
    profit.addItem("Profit Both");
    rtrPanel.add(profit);

    JPanel optionPanel = new JPanel();
    optionPanel.setLayout(new BoxLayout(optionPanel, BoxLayout.X_AXIS));
    optionPanel.add(adaptive);
    optionPanel.add(evolution);
    rtrPanel.add(optionPanel);
 
    JPanel peerPanel          = new JPanel();
    JPanel filterPanel        = new JPanel();
    JPanel fudgePanel         = new JPanel();
    JPanel probBuyPanel       = new JPanel();
    JPanel fracReturnPanel    = new JPanel();
    JPanel bargainFactorPanel = new JPanel();

    peerPanel.setLayout(new BoxLayout(peerPanel, BoxLayout.X_AXIS));
    peerPanel.setBorder(BorderFactory.createEtchedBorder());
    peerPanel.add(new JLabel("Peers     "));
    peerPanel.add(peers);
    peerPanel.add(new JLabel("Neighbors "));
    peerPanel.add(neighbors);
    rtrPanel.add(peerPanel);

    filterPanel.setLayout(new BoxLayout(filterPanel, BoxLayout.X_AXIS));
    filterPanel.setBorder(BorderFactory.createTitledBorder("FilterLevel"));
    filterPanel.add(filter);
    filterPanel.add(filterRatio);
    rtrPanel.add(filterPanel);

    fudgePanel.setLayout(new BoxLayout(fudgePanel, BoxLayout.X_AXIS));
    fudgePanel.setBorder(BorderFactory.createTitledBorder("FudgeFactor"));
    fudgePanel.add(fudge);
    fudgePanel.add(fudgeList);
    fudgePanel.add(fudgeRatio);
    rtrPanel.add(fudgePanel);

    probBuyPanel.setLayout(new BoxLayout(probBuyPanel, BoxLayout.X_AXIS));
    probBuyPanel.setBorder(BorderFactory.createEtchedBorder());
    probBuyPanel.add(new JLabel("Buy Probability "));
    probBuyPanel.add(probBuy);
    rtrPanel.add(probBuyPanel);

    fracReturnPanel.setLayout(new BoxLayout(fracReturnPanel, BoxLayout.X_AXIS));
    fracReturnPanel.setBorder(BorderFactory.createEtchedBorder());
    fracReturnPanel.add(new JLabel("Fraction Return "));
    fracReturnPanel.add(fracReturn);
    rtrPanel.add(fracReturnPanel);

    bargainFactorPanel.setLayout(new BoxLayout(bargainFactorPanel, BoxLayout.X_AXIS));
    bargainFactorPanel.setBorder(BorderFactory.createEtchedBorder());
    bargainFactorPanel.add(new JLabel("Bargain Factor "));
    bargainFactorPanel.add(bargainFactor);
    rtrPanel.add(bargainFactorPanel);

    rtr.addActionListener(listener);
    group.add(rtr);

    rtrPanel.setBorder(BorderFactory.createTitledBorder("RTR Network"));

    return rtrPanel;
  }

  /**
   * Initialize display to setup content distribution.
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel content(ActionListener listener, ButtonGroup group)
  {
    JToggleButton content = new JToggleButton(CONTENT);
    JPanel        panel   = new JPanel();
    JPanel        p1      = new JPanel();
    JPanel        p2      = new JPanel();
    JPanel        p3      = new JPanel();
    JComboBox     combo   = new JComboBox();
    JTextField    cats    = new JTextField("20");
    JTextField    minCats = new JTextField("4");

    swingComponents_.put(DISTRIBUTION, combo);
    swingComponents_.put(CATEGORIES, cats);
    swingComponents_.put(MIN_CATEGORIES, minCats);
     
    panel.setLayout(new GridLayout(5,1,10,10));
    panel.setPreferredSize(new Dimension(100, 100));
    panel.setMaximumSize(new Dimension(100, 100));

    p1.setBorder(BorderFactory.createEtchedBorder());
    p2.setBorder(BorderFactory.createEtchedBorder());
    p3.setBorder(BorderFactory.createEtchedBorder());

    panel.add(p1);
    panel.add(p2);
    panel.add(p3);

    p1.setLayout(new BorderLayout());
    p1.add(new JLabel("Content Distribution Model"), BorderLayout.NORTH);
    combo.addItem("SAROIU");
    combo.addItem("UNIFORM RANDOM");
    p1.add(combo, BorderLayout.CENTER);

    p2.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 0));
    p2.add(new JLabel("Total Categories"));
    p2.add(cats);

    p3.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 0));
    p3.add(new JLabel("Min Categories"));
    p3.add(minCats);
    
    content.addActionListener(listener);
    group.add(content);

    panel.setPreferredSize(new Dimension(200, 170));
    panel.setMinimumSize(new Dimension(200, 170));
    panel.setMaximumSize(new Dimension(200, 170));
    panel.setBorder(BorderFactory.createTitledBorder("Peer Content Setup"));
    return panel;
  }

  /**
   * Initialize display to setup Peer Properties
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel goodBehavior(ActionListener listener, ButtonGroup group)
  {
    JToggleButton g_behavior = new JToggleButton(G_BEHAVIOR);
    JPanel        gp         = new JPanel();
    JPanel        gp1        = new JPanel();
    JPanel        gp2        = new JPanel();
    JPanel        gp3        = new JPanel();
    JPanel        gp4        = new JPanel();

    JComboBox  combo    = null;
    JTextField input    = null;

    gp.setLayout(new GridLayout(5,1,5,5));

    gp1.setLayout(new BorderLayout());
    gp1.setBorder(BorderFactory.createEtchedBorder());
    gp2.setLayout(new BorderLayout());
    gp2.setBorder(BorderFactory.createEtchedBorder());
    gp3.setLayout(new FlowLayout());
    gp3.setBorder(BorderFactory.createEtchedBorder());
    gp4.setLayout(new FlowLayout());
    gp4.setBorder(BorderFactory.createEtchedBorder());

    gp1.add(new JLabel("Query Response Selection"), BorderLayout.NORTH);
    combo = new JComboBox();
    combo.addItem("Random");
    combo.addItem("Global Trust Deterministic");
    combo.addItem("Global Trust Probabilistic");
    combo.addItem("Local Trust Probabilistic");
    combo.addItem("Query Response Time");
    gp1.add(combo, BorderLayout.CENTER);
    swingComponents_.put(GP_RESPONSE, combo);
    gp.add(gp1);

    gp2.add(new JLabel("Message TTL"), BorderLayout.NORTH);
    combo = new JComboBox();
    combo.addItem("Max TTL");
    combo.addItem("Med TTL");
    combo.addItem("Min TTL");
    combo.setSelectedIndex(1);
    gp2.add(combo, BorderLayout.CENTER);
    swingComponents_.put(GP_TTL, combo);
    gp.add(gp2);

    gp3.add(new JLabel("Error Fraction"));
    input = new JTextField("0.0");
    input.setPreferredSize(new Dimension(50, 20));
    input.setMinimumSize(new Dimension(50, 20));
    input.setMaximumSize(new Dimension(50, 20));
    swingComponents_.put(GP_ERR_FRAC, input);
    gp3.add(input);
    gp.add(gp3);

    gp4.add(new JLabel("Freerider Fraction"));
    input = new JTextField("0.0");
    input.setPreferredSize(new Dimension(50, 20));
    input.setMinimumSize(new Dimension(50, 20));
    input.setMaximumSize(new Dimension(50, 20));
    swingComponents_.put(GP_FREERIDER_FRAC, input);
    gp4.add(input);
    gp.add(gp4);

    g_behavior.addActionListener(listener);
    group.add(g_behavior);

    gp.setPreferredSize(new Dimension(200, 130));
    gp.setMinimumSize(new Dimension(200, 130));
    gp.setMaximumSize(new Dimension(200, 130));
    gp.setBorder(BorderFactory.createTitledBorder("Good Peer Behavior"));
    return gp;
  }

  /**
   * Initialize display to setup Peer Properties
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel highTrustBehavior(ActionListener listener, ButtonGroup group)
  {
    JToggleButton h_behavior = new JToggleButton(H_BEHAVIOR);
    JPanel        htp        = new JPanel();
    JPanel        hp1        = new JPanel();
    JPanel        hp2        = new JPanel();
    JPanel        hp3        = new JPanel();

    JComboBox  combo    = null;
    JTextField input    = null;

    htp.setLayout(new GridLayout(5,1,10,10));

    hp1.setLayout(new BorderLayout());
    hp1.setBorder(BorderFactory.createEtchedBorder());
    hp2.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 0));
    hp2.setBorder(BorderFactory.createEtchedBorder());

    hp1.add(new JLabel("Content Category Preference"), BorderLayout.NORTH);
    combo = new JComboBox();
    combo.addItem("Top Categories");
    combo.addItem("Bottom Categories");
    combo.addItem("Random");
    swingComponents_.put(HTP_CAT_PREF, combo);
    hp1.add(combo, BorderLayout.CENTER);
    htp.add(hp1);

    hp2.add(new JLabel("Error Fraction"));
    input = new JTextField("0.0");
    swingComponents_.put(HTP_ERR_FRAC, input);
    hp2.add(input);
    htp.add(hp2);

    h_behavior.addActionListener(listener);
    group.add(h_behavior);

    htp.setPreferredSize(new Dimension(200, 130));
    htp.setMinimumSize(new Dimension(200, 130));
    htp.setMaximumSize(new Dimension(200, 130));
    htp.setBorder(BorderFactory.createTitledBorder("Highly Trusted Peer Behavior"));

    return htp;
  }

  /**
   * Initialize display to setup Peer Properties
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel maliciousBehavior(ActionListener listener, ButtonGroup group)
  {
    JToggleButton m_behavior = new JToggleButton(M_BEHAVIOR);
    JPanel        mp         = new JPanel();
    JPanel        mp1        = new JPanel();
    JPanel        mp2        = new JPanel();
    JPanel        mp3        = new JPanel();

    JComboBox  combo    = null;
    JTextField input    = null;

    mp.setLayout(new GridLayout(5,1,10,10));

    mp1.setLayout(new BorderLayout());
    mp1.setBorder(BorderFactory.createEtchedBorder());
    mp2.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 0));
    mp2.setBorder(BorderFactory.createEtchedBorder());

    mp1.add(new JLabel("Content Category Preference"), BorderLayout.NORTH);
    combo = new JComboBox();
    combo.addItem("Top Categories");
    combo.addItem("Bottom Categories");
    combo.addItem("Random");
    swingComponents_.put(MP_CAT_PREF, combo);
    mp1.add(combo, BorderLayout.CENTER);
    mp.add(mp1);

    mp2.add(new JLabel("Error Fraction"));
    input = new JTextField("0");
    swingComponents_.put(MP_ERR_FRAC, input);
    mp2.add(input);
    mp.add(mp2);

    m_behavior.addActionListener(listener);
    group.add(m_behavior);

    mp.setPreferredSize(new Dimension(200, 130));
    mp.setMinimumSize(new Dimension(200, 130));
    mp.setMaximumSize(new Dimension(200, 130));
    mp.setBorder(BorderFactory.createTitledBorder("Malicious Peer Behavior"));

    return mp;
  }

  /**
   * Initialize display to setup peer activity.
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel activity(ActionListener listener, ButtonGroup group)
  {
    JToggleButton activity = new JToggleButton(ACTIVITY);
    JPanel        panel    = new JPanel();
    JPanel        p1       = new JPanel();
    JComboBox     combo    = new JComboBox();

    combo.addItem("Always up");
    combo.addItem("Always up no queries");
    combo.addItem("Random uptime");
    panel.add(combo);
    swingComponents_.put(ACTIVITY, combo);
    
    activity.addActionListener(listener);
    group.add(activity);

    panel.setPreferredSize(new Dimension(200, 100));
    panel.setMinimumSize(new Dimension(200, 100));
    panel.setMaximumSize(new Dimension(200, 100));
    panel.setBorder(BorderFactory.createTitledBorder("Peer Activity"));
    return panel;
  }

  /**
   * Initialize display to setup collective.
   * @param simulator the simulator
   * @param frame the frame
   */
  private JPanel collective(ActionListener listener, ButtonGroup group)
  {
    JToggleButton collective = new JToggleButton(COLLECTIVE);
    JPanel        panel      = new JPanel();
    JPanel        p1         = new JPanel();
    JPanel        p2         = new JPanel();
    JComboBox     combo      = null;
 
    panel.setLayout(new GridLayout(5,1,10,10));
    p1.setLayout(new BorderLayout());
    p1.setBorder(BorderFactory.createEtchedBorder());

    p2.setLayout(new BorderLayout());
    p2.setBorder(BorderFactory.createEtchedBorder());

    panel.add(p1);
    panel.add(p2);

    p1.add(new JLabel("Collective Type"), BorderLayout.NORTH);
    combo = new JComboBox();
    combo.addItem("Malicious Collective");

    if (parent_.mode() == QueryCyclePanel.MODE_ET || 
        parent_.mode() == QueryCyclePanel.MODE_ALL)
    {
      combo.addItem("Camouflaged Collective");
      combo.addItem("Spy Collective");
    }

    if (parent_.mode() == QueryCyclePanel.MODE_APT || 
        parent_.mode() == QueryCyclePanel.MODE_ALL)
    {
      combo.addItem("Hidden Collective A");
      combo.addItem("Hidden Collective B");
      combo.addItem("Hidden Spy Collective");
    }

    p1.add(combo, BorderLayout.CENTER);
    swingComponents_.put(COLLECTIVE_TYPE, combo);

    p2.add(new JLabel("Trust Value Configuration"), BorderLayout.NORTH);
    combo = new JComboBox();
    combo.addItem("Chain");
    combo.addItem("Clique");
    combo.addItem("Random");
    p2.add(combo, BorderLayout.CENTER);
    swingComponents_.put(COLLECTIVE_TRUST, combo);

    collective.addActionListener(listener);
    group.add(collective);

    panel.setPreferredSize(new Dimension(200, 150));
    panel.setMinimumSize(new Dimension(200, 150));
    panel.setMaximumSize(new Dimension(200, 150));
    panel.setBorder(BorderFactory.createTitledBorder("Setup Malicious Collective"));

    return panel;
  }

// ===================================================================

  private void initRTR(FileOutputStream out)
  {
    JTextField    peers         = (JTextField)swingComponents_.get(RTR_PEERS); 
    JTextField    neighbors     = (JTextField)swingComponents_.get(RTR_NEIGHBORS); 
    JComboBox     rtrType       = (JComboBox)swingComponents_.get(RTR_TYPE); 
    JTextField    filter        = (JTextField)swingComponents_.get(RTR_FILTER); 
    JTextField    filterRatio   = (JTextField)swingComponents_.get(RTR_FILTER_RATIO); 
    JTextField    fudge         = (JTextField)swingComponents_.get(RTR_FUDGE); 
    JTextField    fudgeList     = (JTextField)swingComponents_.get(RTR_FUDGE_LIST); 
    JTextField    fudgeRatio    = (JTextField)swingComponents_.get(RTR_FUDGE_RATIO); 
    JTextField    probBuy       = (JTextField)swingComponents_.get(RTR_PROBBUY); 
    JTextField    fracReturn    = (JTextField)swingComponents_.get(RTR_FRACRETURN); 
    JTextField    bargainFactor = (JTextField)swingComponents_.get(RTR_BARGAINFACTOR); 

    JComboBox     profit        = (JComboBox)swingComponents_.get(RTR_PROFIT); 
    JCheckBox     adaptive      = (JCheckBox)swingComponents_.get(RTR_ADAPTIVE); 
    JCheckBox     evolution     = (JCheckBox)swingComponents_.get(RTR_EVOLUTION); 

    JTextField goodPeers = (JTextField) swingComponents_.get(NUM_GP);
    goodPeers.setText("0"); // Turn off regular mode.

    int pType = PeerManagerRTR.PROFIT_BOTH;
    if (profit.equals("Profit Seller"))
      pType = PeerManagerRTR.PROFIT_SELLER;
    else if (profit.equals("Profit Buyer"))
      pType = PeerManagerRTR.PROFIT_BUYER;
    else if (profit.equals("Profit Both"))
      pType = PeerManagerRTR.PROFIT_BOTH;

    int rType = PeerBehavior.GOOD_PEER;
    if (rtrType.equals("Competitive Forward"))
      rType = PeerBehavior.COMPETITIVE_FORWARD_PEER;
    else if (rtrType.equals("All Forward"))
      rType = PeerBehavior.NO_FORWARD_PEER;
    else if (rtrType.equals("No Forward"))
      rType = PeerBehavior.ALL_FORWARD_PEER;

    if (out == null)
    {
      PeerManagerRTR rtrManager;
      parent_.manager().rtrPeers(Integer.parseInt(peers.getText()), 
                                 Integer.parseInt(neighbors.getText()));

      rtrManager = (PeerManagerRTR) parent_.manager().peerManager();
      rtrManager.rtrType(rType);
      rtrManager.filterLevel(QueryCycleScriptParser.parseCommand(filter.getText()),
                             QueryCycleScriptParser.parseCommand(filterRatio.getText()));
      rtrManager.fudgeFactor(Integer.parseInt(fudge.getText()),
                             QueryCycleScriptParser.parseCommand(fudgeList.getText()),
                             QueryCycleScriptParser.parseCommand(fudgeRatio.getText()));
      
      rtrManager.probBuy(QueryCycleScriptParser.parseCommand(probBuy.getText()));
      rtrManager.fracReturn(Double.parseDouble(fracReturn.getText()));
      rtrManager.bargainFactor(Double.parseDouble(fracReturn.getText()));

      rtrManager.profitType(pType);

      if (adaptive.isSelected())
        rtrManager.adaptiveTopology(1);
      else
        rtrManager.adaptiveTopology(0);

      if (evolution.isSelected())
        rtrManager.evolution(true);
      else
        rtrManager.evolution(false);

      view_ = new InitializationView(parent_.manager()); 

      displayPanel_.removeAll();
      displayControlPanel_.removeAll();

      displayPanel_.add( view_ );
      displayControlPanel_.add( view_.controlPanel() );

      validate();
      repaint();
    }
    else
    {
      String output;

      try { 
        output = "rtrPeers " + peers.getText() + " " + neighbors.getText() + "\n" +
	       "rtrType " + rType + "\n" +
                 "filterLevel\n" + filter.getText() + "\n" + filterRatio.getText() + "\n" +
	       "fudgeFactor " + fudge.getText() + "\n" + 
                                  fudgeList.getText() + "\n" + 
                                  fudgeRatio.getText() + "\n" +
                 "probBuy " + probBuy.getText() + "\n" +
                 "fracReturn " + fracReturn.getText() + "\n" +
                 "bargainFactor " + bargainFactor.getText() + "\n" +
	       "profitType " + pType + "\n";

        if (adaptive.isSelected()) output += "adaptive\n";
        if (evolution.isSelected()) output += "evolution\n";

        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  private void initPeers(FileOutputStream out)
  {
    JTextField gpf   = (JTextField) swingComponents_.get(NUM_GP);
    JTextField htpf  = (JTextField) swingComponents_.get(NUM_HTP);
    JTextField mpf   = (JTextField) swingComponents_.get(NUM_MP);
    JTextField gpnf  = (JTextField) swingComponents_.get(NUM_GPN);
    JTextField htpnf = (JTextField) swingComponents_.get(NUM_HTPN);
    JTextField mpnf  = (JTextField) swingComponents_.get(NUM_MPN);

    int peers = Integer.parseInt(gpf.getText());
    if (peers == 0) 
      return;  // No peers entered.

    if (out == null)
    {
      parent_.manager().peers(Integer.parseInt(gpf.getText()),
                              Integer.parseInt(gpnf.getText()),
                              Integer.parseInt(mpf.getText()),
                              Integer.parseInt(mpnf.getText()),
                              Integer.parseInt(htpf.getText()),
                              Integer.parseInt(htpnf.getText()));

      view_ = new InitializationView(parent_.manager()); 

      displayPanel_.removeAll();
      displayControlPanel_.removeAll();

      displayPanel_.add( view_ );
      displayControlPanel_.add( view_.controlPanel() );

      validate();
      repaint();
    }
    else
    {
      String output = "peers " + gpf.getText()  + " " + gpnf.getText() + " " +
                                 mpf.getText()  + " " + mpnf.getText() + " " +
                                 htpf.getText() + " " + htpnf.getText() + "\n"; 
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  private void initContent(FileOutputStream out)
  {
    JComboBox  distribution = (JComboBox) swingComponents_.get(DISTRIBUTION);
    JTextField cats         = (JTextField) swingComponents_.get(CATEGORIES);
    JTextField minCats      = (JTextField) swingComponents_.get(MIN_CATEGORIES);

    String selectedDistribution = (String) distribution.getSelectedItem();

    int dist = 0;
    if (selectedDistribution.equals("SAROIU"))
      dist = PeerContentManager.SAROIU;
    else if (selectedDistribution.equals("UNIFORM RANDOM"))
      dist = PeerContentManager.UNIFORMRANDOM;

    if (out == null)
    {
      parent_.manager().content(Integer.parseInt(cats.getText()),
                                Integer.parseInt(minCats.getText()), dist);
    }
    else
    {
      String output = "content " + cats.getText() + " " + minCats.getText() +
                                   " " + dist + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  private void initHTPBehavior(FileOutputStream out)
  {
    JComboBox  catPref  = (JComboBox) swingComponents_.get(HTP_CAT_PREF);
    JTextField errFrac  = (JTextField) swingComponents_.get(HTP_ERR_FRAC);

    String preference = (String) catPref.getSelectedItem();

    int pref = 0;
    if (preference.equals("Top Categories"))
      pref = PeerContentManager.TOP_CATEGORY;
    else if (preference.equals("Botton Categories"))
      pref = PeerContentManager.BOTTOM_CATEGORY;
    else if (preference.equals("Random"))
      pref = PeerContentManager.RANDOM_CATEGORY;

    if (out == null)
    {
      parent_.manager().highTrustPeerCategoryPref(pref);
      parent_.manager().errorFraction(Double.parseDouble(errFrac.getText()), 
                                      PeerBehavior.HIGHLY_TRUSTED_PEER);
    }
    else
    {
      String output = "highTrustPeerCategory " + pref + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
      output = "errorFraction " + errFrac.getText() + " " +
               PeerBehavior.HIGHLY_TRUSTED_PEER + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  private void initMPBehavior(FileOutputStream out)
  {
    JComboBox  catPref  = (JComboBox) swingComponents_.get(MP_CAT_PREF);
    JTextField errFrac  = (JTextField) swingComponents_.get(MP_ERR_FRAC);

    String preference = (String) catPref.getSelectedItem();

    int pref = 0;
    if (preference.equals("Top Categories"))
      pref = PeerContentManager.TOP_CATEGORY;
    else if (preference.equals("Botton Categories"))
      pref = PeerContentManager.BOTTOM_CATEGORY;
    else if (preference.equals("Random"))
      pref = PeerContentManager.RANDOM_CATEGORY;

    if (out == null)
    {
      parent_.manager().maliciousPeerCategoryPref(pref);
      parent_.manager().errorFraction(Double.parseDouble(errFrac.getText()), 
                                      PeerBehavior.MALICIOUS_PEER);
    }
    else
    {
      String output = "maliciousPeerCategoryPref " + pref + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
      output = "errorFraction " + errFrac.getText() + " " +
               PeerBehavior.MALICIOUS_PEER + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  private void initGPBehavior(FileOutputStream out)
  {
    JComboBox  combo   = (JComboBox) swingComponents_.get(GP_RESPONSE);
    JTextField errFrac = (JTextField) swingComponents_.get(GP_ERR_FRAC);
    JTextField fr      = (JTextField) swingComponents_.get(GP_FREERIDER_FRAC);

    String response = (String) combo.getSelectedItem();

    int selector = 0;
    if (response.equals("Random"))
      selector = PeerBehavior.RPB_RANDOM;
    else if (response.equals("Global Trust Deterministic"))
      selector = PeerBehavior.RPB_TRUSTDETERMINISTIC;
    else if (response.equals("Global Trust Probabilistic"))
      selector = PeerBehavior.RPB_TRUSTPROBABILISTIC;
    else if (response.equals("Local Trust Probabilistic"))
      selector = PeerBehavior.RPB_SIMPLEPROBABILISTIC;
    else if (response.equals("Query Response Time"))
      selector = PeerBehavior.RPB_RESPONSETIME;

    if (out == null)
    {
      parent_.manager().goodPeerResponseSelector(selector);
      parent_.manager().errorFraction(Double.parseDouble(errFrac.getText()), 
                                      PeerBehavior.GOOD_PEER);
      parent_.manager().freeriderFraction(Double.parseDouble(fr.getText()));
    }
    else
    {
      String output = "goodPeerResponseSelector " + selector + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
      output = "errorFraction " + errFrac.getText() + " " + 
               PeerBehavior.GOOD_PEER + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }

      output = "freeriderFraction " + fr.getText() + "\n"; 
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }

    combo    = (JComboBox) swingComponents_.get(GP_TTL);
    response = (String) combo.getSelectedItem();
    selector = PeerMsg.MED_TTL;
    if (response.equals("Max TTL"))
      selector = PeerMsg.MAX_TTL;
    else if (response.equals("Med TTL"))
      selector = PeerMsg.MED_TTL;
    else if (response.equals("Min TTL"))
      selector = PeerMsg.MIN_TTL;

    if (out == null)
    {
      parent_.manager().goodPeerTTL(selector);
    }
    else
    {
      String output = "goodPeerTTL " + selector + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
  }

  private void initActivity(FileOutputStream out)
  {
    JComboBox combo = (JComboBox) swingComponents_.get(ACTIVITY);

    String activity = (String) combo.getSelectedItem();

    int act = 0;
    if (activity.equals("Always up"))
      act = PeerBehavior.ACTIVITY_ALWAYSUP;
    else if (activity.equals("Always up no queries"))
      act = PeerBehavior.ACTIVITY_ALWAYSUPNOQUERIES;
    else if (activity.equals("Random uptime"))
      act = PeerBehavior.ACTIVITY_RANDOM;

    if (view_ == null) return;

    Vector selected = view_.selectedPeers();

    if (out == null)
    {
      parent_.manager().setPeerActivity(selected, act);

      validate();
      repaint();
    }
    else if (selected.size() > 0)
    {
      String output = "setPeerActivity ";
      for (int i = 0; i < selected.size(); i++)
        output += selected.elementAt(i).toString() + " ";
      output += act + "\n";
      try { 
        out.write( output.getBytes() );
      } catch (Exception e) {
        System.err.println(e);
      }
    }
    view_.clearSelectedPeers();
  }

  private void initCollective(FileOutputStream out)
  {
    JComboBox c1  = (JComboBox) swingComponents_.get(COLLECTIVE_TYPE);
    JComboBox c2 = (JComboBox) swingComponents_.get(COLLECTIVE_TRUST);

    String type  = (String) c1.getSelectedItem();
    String trust = (String) c2.getSelectedItem();

    int ct = 0;
    if (type.equals("Malicious Collective"))
      ct = PeerBehavior.MALICIOUS_COLLECTIVE;
    else if (type.equals("Camouflaged Collective"))
      ct = PeerBehavior.MALICIOUS_CAMOUFLAGED;
    else if (type.equals("Spy Collective"))
      ct = PeerBehavior.MALICIOUS_SPY;
    else if (type.equals("Hidden Collective A"))
      ct = PeerBehavior.MALICIOUS_HIDDEN_A;
    else if (type.equals("Hidden Collective B"))
      ct = PeerBehavior.MALICIOUS_HIDDEN_B;
    else if (type.equals("Hidden Spy Collective"))
      ct = PeerBehavior.MALICIOUS_HIDDEN_SPY;

    int tt = 0;
    if (type.equals("Chain"))
      tt = PeerManager.MC_CHAIN;
    else if (type.equals("Clique"))
      tt = PeerManager.MC_CLIQUE;
    else if (type.equals("Random"))
      tt = PeerManager.MC_RANDOM;

    if (view_ == null) return;

    Vector selected = view_.selectedPeers();

    if (out == null)
    {
      collectives_.addElement(selected.clone());
      parent_.manager().establishCollective(selected, ct, tt);

      validate();
      repaint();
    }
    else
    {
      if (selected != null && selected.size() > 0)
        collectives_.addElement(selected.clone());

      for (Enumeration enum = collectives_.elements(); enum.hasMoreElements(); )
      {
        selected = (Vector) enum.nextElement();
        String output = "establishCollective ";
        for (int i = 0; i < selected.size(); i++)
          output += selected.elementAt(i).toString() + " ";
        output += ct + " " + tt + "\n";
        try { 
          out.write( output.getBytes() );
        } catch (Exception e) {
          System.err.println(e);
        }
      }
    }
    view_.clearSelectedPeers();
  }

  /**
   * Initialize frame to load simulator display
   * @param frame the frame
   */
  private void loadSimulation()
  {
    JComboBox combo = (JComboBox) swingComponents_.get(LOAD_SIMULATOR);
    String    file  = (String)savedSims_.get(combo.getSelectedItem());
    URL url = null;

    try {
      if (parent_.container() instanceof JApplet)
      {
        JApplet applet = (JApplet) parent_.container();
        url = new URL(applet.getCodeBase(), parent_.simulationsDir()+"/"+file); 
      }
      else
      {
        File f = new File(parent_.simulationsDir()+"/"+file);
        url = f.toURL();
      }

      InputStream       in        = url.openStream(); 
      ObjectInputStream objstream = new ObjectInputStream( in );
      parent_.manager((QueryCycleManager) objstream.readObject());
      parent_.initializationComplete();
    } catch (Exception e) {
      System.err.println("LOAD SIMULATOR: " + e); 
      e.printStackTrace(); 
    }
  }

}
