/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.font.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import java.net.*;


import qcsim.*;
import qcsim.peer.*;
import qcsim.peer.content.*;

 /*
  * Title: Simulator Manager Implementation
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Create and initialize simulation.
  * @author unascribed
  * @version 1.0
  */

public class QueryCyclePanel extends JPanel
{
  public static final int MODE_ALL = 0;
  public static final int MODE_ET  = 1;
  public static final int MODE_APT = 2;
  public static final int MODE_RTR = 3;

  private int mode_;

  private QueryCycleManager manager_;

  private Container container_;

  private String resource_;

  public static void main(String [] args)
  {
    JFrame frame = new JFrame("Query Cycle Simulator");

    if (args.length == 0)
    {
      System.err.println("USAGE: QueryCyclePanel simulations [mode]");
      System.exit(1);
    }

    String resource = args[0];

    int mode = MODE_ALL;
    if (args.length == 2)
    {
      if (args[1].equals("eigentrust"))    mode = MODE_ET;
      else if (args[1].equals("adaptive")) mode = MODE_APT;
      else if (args[1].equals("rtr"))      mode = MODE_RTR;
      else                                 mode = MODE_ALL;
    }

    QueryCyclePanel manager = new QueryCyclePanel(frame, resource, MODE_ALL);
    frame.setContentPane(manager);
    frame.setSize(800, 700);
    frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    
    frame.setVisible(true);
  }

  /**
   * Constructor
   */
  public QueryCyclePanel(Container container, String resource, int mode)
  {
    mode_      = mode;
    resource_  = resource;
    container_ = container;
    manager_   = new qcsim.impl.QueryCycleManagerImpl(resource);

    add(new InitializationPanel(this));

    validate();
    repaint();
  }

  public int mode()
  {
    return mode_;
  }

  public Dimension dimension()
  {
    return new Dimension(800, 700);
  }

  public QueryCycleManager manager()
  {
    return manager_;
  }

  public void manager(QueryCycleManager manager)
  {
    manager_ = manager;
  }

  public Container container()
  {
    return container_;
  }

  public String simulationsDir()
  {
    return resource_ + "/simulations";
  }

  public String scriptsDir()
  {
    return resource_ + "/scripts";
  }

  public String graphDatDir()
  {
    return resource_ + "/graphDat";
  }

  public void initializationComplete()
  {
    removeAll();
    add(new SimulationPanel(this));
    validate();
    repaint();
  }
  

  public void simulationComplete()
  {
    removeAll();
    manager_ = new qcsim.impl.QueryCycleManagerImpl(resource_);
    add(new InitializationPanel(this));
    validate();
    repaint();
  }

}
