/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.font.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import java.net.*;

import qcui.view.*;

import qcsim.*;
import qcsim.peer.*;
import qcsim.peer.content.*;

 /*
  * Title: Simulator Manager Implementation
  * Description: P2P file sharing network simulator
  * Copyright: Copyright (c) 2002
  * Company:
  */

 /**
  * Create and initialize simulation.
  * @author unascribed
  * @version 1.0
  */

public class SimulationPanel extends JPanel 
{
  public static final String NEXT_CYCLE = "Next Cycle";
  public static final String PREV_CYCLE = "Previous Cycle";
  public static final String AUTO_CYCLE = "Auto Cycle";
  public static final String HALT_CYCLE = "Halt Cycle";
  public static final String SAVE       = "Save";
  public static final String DONE       = "Done";
  public static final String RESET      = "Reset";
  public static final String VIEW       = "View";


  private String currentView_;

  private AutoDisplay autoDisplay_;

  private QueryCyclePanel  parent_;

  private JPanel  controlPanel_;
  private JPanel  displayPanel_;

  private JLabel cycle_;
  private int    currentCycle_;

  private Hashtable views_;

  /**
   * Constructor
   */
  public SimulationPanel(QueryCyclePanel parent)
  {
    parent_ = parent;
    views_  = new Hashtable();

    setLayout(new BorderLayout());
    setPreferredSize(parent.dimension());
    setMinimumSize(parent.dimension());
    setMaximumSize(parent.dimension());

    controlPanel_ = new JPanel();
    displayPanel_ = new JPanel();
    add(controlPanel_, BorderLayout.WEST);
    add(displayPanel_, BorderLayout.CENTER);

    setupPanel();

    validate();
    repaint();
  }

  public QueryCycleView view(String view)
  {
    return (QueryCycleView) views_.get(view);
  }

  private void setupPanel()
  {
    displayPanel_.setLayout(new CardLayout()); 

    controlPanel_.setLayout(new BoxLayout(controlPanel_, BoxLayout.Y_AXIS)); 
    controlPanel_.setBorder(BorderFactory.createTitledBorder("Simulation Control Panel"));
    controlPanel_.setPreferredSize(new Dimension(200, getHeight()-50 ));
    controlPanel_.setMinimumSize(new Dimension(200, getHeight()-50 ));
    controlPanel_.setMaximumSize(new Dimension(200, getHeight()-50 ));


    JPanel commands = new JPanel();
    commands.setPreferredSize(new Dimension(200, 300 ));
    commands.setMinimumSize(new Dimension(200, 300 ));
    commands.setMaximumSize(new Dimension(200, 300 ));
    commands.setLayout(new GridLayout(10,1)); 
    commands.setBorder(BorderFactory.createTitledBorder("Simulation Commands"));
    
    cycle_ = new JLabel("");    
    commands.add(cycle_);

    ActionListener listener = new ActionListener() {
      public void actionPerformed(ActionEvent event) 
      {
        String cmd = event.getActionCommand();
        if (!cmd.equals(HALT_CYCLE))
        {
          // Only this thread may change autoDisplay_ (so no race condition).
          if (autoDisplay_ != null && autoDisplay_.mode() == AutoDisplay.AUTO) 
            return;
        }
        command(cmd);

        if (cmd.equals(HALT_CYCLE))
          ((JButton)event.getSource()).setText(AUTO_CYCLE);
        else if (cmd.equals(AUTO_CYCLE))
          ((JButton)event.getSource()).setText(HALT_CYCLE);

      }
    };

    JButton   button = null;

    button = new JButton(NEXT_CYCLE);
    button.addActionListener(listener);
    commands.add(button);

    button = new JButton(PREV_CYCLE);
    button.addActionListener(listener);
    commands.add(button);

    button = new JButton(AUTO_CYCLE);
    button.addActionListener(listener);
    commands.add(button);

    button = new JButton(RESET);
    button.addActionListener(listener);
    commands.add(button);

    button = new JButton(SAVE);
    button.addActionListener(listener);
    commands.add(button);

    button = new JButton(DONE);
    button.addActionListener(listener);
    commands.add(button);

    button = new JButton(VIEW);
    commands.add(button);

    controlPanel_.add(commands);
    controlPanel_.add(setupViewPanel(button));
  }

  private JPanel setupViewPanel(JButton viewButton)
  {
    final JPanel viewPanel = new JPanel();
    viewPanel.setPreferredSize(new Dimension(200, 300 ));
    viewPanel.setMinimumSize(new Dimension(200, 300 ));
    viewPanel.setMaximumSize(new Dimension(200, 300 ));
    viewPanel.setLayout(new CardLayout());

    JPanel viewCommandPanel = new JPanel();
    viewCommandPanel.setLayout( new GridLayout(8,1) );
    viewCommandPanel.setBorder(BorderFactory.createTitledBorder("Simulation Views"));
    viewPanel.add(viewCommandPanel, viewButton.getText());
    viewButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) 
      {
        String     cmd   = event.getActionCommand();
        CardLayout cards = (CardLayout) viewPanel.getLayout();
        cards.show(viewPanel, cmd);
      }
    });

    ActionListener listener = new ActionListener() {
      public void actionPerformed(ActionEvent event) 
      {
        String     cmd   = event.getActionCommand();
        CardLayout cards = (CardLayout) viewPanel.getLayout();
        cards.show(viewPanel, cmd);

        cards = (CardLayout) displayPanel_.getLayout();
        cards.show(displayPanel_, cmd);

        currentView_ = cmd;
      }
    };

    JButton viewCommand = null;

    FileShareView fsv = new FileShareView(parent_.manager());
    viewCommand = new JButton("File Share View");
    viewCommand.addActionListener( listener );
    viewCommandPanel.add(viewCommand);
    displayPanel_.add(fsv, viewCommand.getText());
    views_.put(viewCommand.getText(), fsv);
    viewPanel.add(fsv.controlPanel(), viewCommand.getText());
    currentView_ = viewCommand.getText();

    if (parent_.mode() == QueryCyclePanel.MODE_ET || 
        parent_.mode() == QueryCyclePanel.MODE_ALL)
    {
      TrustView trv = new TrustView(parent_.manager(), parent_.graphDatDir());
      viewCommand = new JButton("Trust View");
      viewCommand.addActionListener( listener );
      viewCommandPanel.add(viewCommand);
      displayPanel_.add(trv, viewCommand.getText());
      views_.put(viewCommand.getText(), trv);
      viewPanel.add(trv.controlPanel(), viewCommand.getText());
    }


    if (parent_.mode() == QueryCyclePanel.MODE_APT || 
        parent_.mode() == QueryCyclePanel.MODE_ALL)
    {
      TopologyView tv = new TopologyView(parent_.manager());
      viewCommand = new JButton("Topology View");
      viewCommand.addActionListener( listener );
      viewCommandPanel.add(viewCommand);
      displayPanel_.add(tv, viewCommand.getText());
      views_.put(viewCommand.getText(), tv);
      viewPanel.add(tv.controlPanel(), viewCommand.getText());
      currentView_ = viewCommand.getText();
   

      LinkGraphView lgv = new LinkGraphView(parent_.manager(), 
                                            parent_.graphDatDir());
      viewCommand = new JButton("Link Graph View");
      viewCommand.addActionListener( listener );
      viewCommandPanel.add(viewCommand);
      displayPanel_.add(lgv, viewCommand.getText());
      views_.put(viewCommand.getText(), lgv);
      viewPanel.add(lgv.controlPanel(), viewCommand.getText());

      ShortestPathView spv = new ShortestPathView(parent_.manager(), 
                                                  parent_.graphDatDir());
      viewCommand = new JButton("Shortest Path View");
      viewCommand.addActionListener( listener );
      viewCommandPanel.add(viewCommand);
      displayPanel_.add(spv, viewCommand.getText());
      views_.put(viewCommand.getText(), spv);
      viewPanel.add(spv.controlPanel(), viewCommand.getText());

      NetworkTrafficView ntv = new NetworkTrafficView(parent_.manager(), parent_.graphDatDir());
      viewCommand = new JButton("Network Traffic View");
      viewCommand.addActionListener( listener );
      viewCommandPanel.add(viewCommand);
      displayPanel_.add(ntv, viewCommand.getText());
      views_.put(viewCommand.getText(), ntv);
      viewPanel.add(ntv.controlPanel(), viewCommand.getText());
    }

    return viewPanel;
  }

  public void command(String command)
  {
    if (command.equals(SAVE))
    {
      saveSimulation();
    }
    else if (command.equals(DONE))
    {
      if (parent_.container() instanceof JFrame)
        parent_.manager().flushPlots();

      parent_.simulationComplete();
    }
    else if (command.equals(HALT_CYCLE))
    {
      autoDisplay_.mode(AutoDisplay.HALT);
    }
    else if (command.equals(AUTO_CYCLE))
    {
      autoDisplay_ = new AutoDisplay( this );
      autoDisplay_.start();
    }
    else if (command.equals(NEXT_CYCLE))
    {
      parent_.manager().cycle(currentCycle_);
      parent_.manager().execPlots();

      currentCycle_++;
      cycle_.setText("Current Cycle: "+currentCycle_);

      validate();
      repaint();
    }
    else if (command.equals(PREV_CYCLE))
    {
      if (currentCycle_ == 1) return;

      currentCycle_--;
      parent_.manager().cycle(currentCycle_ - 1);
      cycle_.setText("Current Cycle: "+currentCycle_);
   
      validate();
      repaint();
    }
    else if (command.equals(RESET))
    {
      currentCycle_ = 0;
      parent_.manager().reset();
      command(NEXT_CYCLE);
    }
  }

  private void saveSimulation()
  {
     if (!(parent_.container() instanceof JFrame)) return;

     final JDialog dialog = new JDialog((JFrame)parent_.container(), "Save");

     final JTextField filename = new JTextField();
     final JTextField desc     = new JTextField();

     JPanel main = new JPanel();
     main.setLayout(new GridLayout(2,2));
     main.add(new JLabel("File Description"));
     main.add(desc);
     main.add(new JLabel("File Name"));
     main.add(filename);

     JPanel  controls = new JPanel();
     JButton save     = new JButton("Save");
     JButton cancel   = new JButton("Cancel");
     controls.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 1));
     controls.add(save);
     controls.add(cancel);

     ActionListener listener = new ActionListener() {
       public void actionPerformed(ActionEvent event) 
       {
         String command = event.getActionCommand();
         if (command.equals("Save"))
           parent_.manager().save(filename.getText(), desc.getText());
         dialog.dispose();
       }
     };

     save.addActionListener(listener);
     cancel.addActionListener(listener);

     dialog.getContentPane().setLayout(new BorderLayout(30, 30));
     dialog.getContentPane().add(main, BorderLayout.CENTER);
     dialog.getContentPane().add(controls, BorderLayout.SOUTH);

     dialog.pack();
     dialog.setVisible(true);
     dialog.toFront();
  } 

  private class AutoDisplay extends Thread
  {
    public static final int AUTO = 0;
    public static final int HALT = 1;
  
    private SimulationPanel parent_;
  
    private int mode_;
  
    public AutoDisplay(SimulationPanel parent)
    {
      parent_ = parent;
      mode_   = AUTO;
    }
  
    public void run()
    {
      synchronized (this)
      {
        parent_.view(currentView_).notifiee( this );
        while( mode_ == AUTO )
        { 
          parent_.command(NEXT_CYCLE);
          try {
            this.wait();
            sleep(1000);
          } catch (Exception e) {}
        }
        parent_.view(currentView_).notifiee( null );
      }
    }
  
    public void mode(int mode)
    {
      mode_ = mode;
    }
  
    public int mode()
    {
      return mode_;
    }
  }

}
