/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui.view;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.basic.*;

import qcsim.*;
import qcsim.peer.*;
import qcsim.util.*;

public class FileShareView extends QueryCycleView 
       implements ActionListener, MouseListener
{
  private JCheckBox colorTrust_;

  private JCheckBox animate_;

  private Peer selectedPeer_;

  private Vector downloads_;

  private int animatedCycle_;

  public FileShareView(QueryCycleManager manager)
  {
    super(manager);
    addMouseListener( this );
    animatedCycle_ = -1;

    selectedPeer_ = null;
    colorTrust_   = new JCheckBox("Color Trust");
    animate_      = new JCheckBox("Animate");

    JPanel checkBoxPanel = new JPanel();
    checkBoxPanel.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 0));
    checkBoxPanel.add(colorTrust_);
    checkBoxPanel.add(animate_);

    controlPanel().add(checkBoxPanel);

    colorTrust_.addActionListener( this );
    animate_.addActionListener( this );

  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent( g );
    super.drawBackground(g);
    controlPanel().setBorder(BorderFactory.createTitledBorder(viewName()));
   
    if (animate_.isSelected() && 
        peerManager().manager().cycle() != animatedCycle_)
    {
      animate( g );
    }
    else
    {
      DownloadDescription download = null;
      PeerDescription     desc     = null;

      if (selectedPeer_ != null)
      {
        desc = selectedPeer_.description();
        desc.draw(g, desc.color(colorTrust_.isSelected()));
        for (Enumeration enum = desc.downloadDescriptions().elements(); 
             enum.hasMoreElements(); )
        {
          download = (DownloadDescription) enum.nextElement();
          download.draw(g, false);            // No animation
          download.drawPeers(g, colorTrust_.isSelected()); // Draw end points.
        }
      } 
      else
      {
        for (int i = 0; i < peers(); i++)
        {
          desc = peer(i).description();
          desc.draw(g, desc.color(colorTrust_.isSelected()));
          for (Enumeration enum = desc.downloadDescriptions().elements(); 
               enum.hasMoreElements(); )
          {
            download = (DownloadDescription) enum.nextElement();
            download.draw(g, false); // No animation
            download.drawPeers(g, colorTrust_.isSelected()); // Draw end points.
          }
        }
      }

      super.notifyDone();
    }
  } 

  public String viewName() 
  { 
    return (selectedPeer_ == null) ? 
           "File Share View: All Peers" :
           "File Share View: Peer "+selectedPeer_.peerID(); 
  }

  public void actionPerformed(ActionEvent event) 
  {
    validate();
    repaint();
  }

  private void animate(Graphics g)
  {
    if (downloads_ == null)
    {
      downloads_ = new Vector();

      if (selectedPeer_ == null)
      {
        for (int i = 0; i < peers(); i++)
        {
          Vector downloads = peer(i).description().downloadDescriptions();

          if (downloads.size() > 0)
            downloads_.addElement(downloads);
        }
      }
      else
      {
        Vector downloads = selectedPeer_.description().downloadDescriptions();
        if (downloads.size() > 0)
          downloads_.addElement(downloads);
      }
    }

    DownloadDescription download  = null;
    for (Enumeration enum = downloads_.elements(); enum.hasMoreElements(); )
    {
      Vector  downloads = (Vector) enum.nextElement();

      download = (DownloadDescription) downloads.elementAt(0);
      download.drawPeers(g, colorTrust_.isSelected()); // Draw end points.

      if (download.draw(g, true)) 
      {
        downloads.remove(download);
      }

      if (downloads.size() == 0) 
      {
        downloads_.remove(downloads);
        enum = downloads_.elements();
      }
    }

    if (downloads_.size() == 0) 
       downloads_ = null;

    if (selectedPeer_ != null)
    {
      String downloadText = "Peer "+download.start_.peerID()+
                            " is uploading file.";
      Integer cycle = new Integer(peerManager().manager().cycle() - 1);

      g.setColor(Color.black);
      g.drawString(downloadText, 20, 20);
      downloadText = "Global Trust: "+
                     Double.toString(download.start_.globalTrust(cycle));
      g.drawString(downloadText, 20, 35);

      try {
        Thread.sleep(10);
      } catch (Exception e) {}
    }
    else
    {
      for (int i = 0; i < peers(); i++)
      {
        PeerDescription d = peer(i).description();
        d.draw(g, d.color(colorTrust_.isSelected()));
      }
      
    }

    if (downloads_ == null) 
    {
      animatedCycle_ = peerManager().manager().cycle();
      super.notifyDone();
    }
    else
    {
      RepaintManager rm = RepaintManager.currentManager(this);
      rm.markCompletelyDirty(this);

      Runnable animate = new Runnable() {
                           public void run() { repaint(); } 
                         };
      SwingUtilities.invokeLater(animate);
    }
  }

  public void mouseClicked(MouseEvent e) 
  {
    selectedPeer_ = null;

    for (int i = 0; i < peers(); i++)
    {
      Peer p = (Peer) peer(i);
      if (p.description().contains(e.getX(), e.getY()))
      {
        selectedPeer_ = p;
        
        break;
      }
    }
    repaint();
  }

  public void mouseEntered(MouseEvent e) {}

  public void mouseExited(MouseEvent e) {} 

  public void mousePressed(MouseEvent e) {}

  public void mouseReleased(MouseEvent e) {}
}
