/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui.view;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.basic.*;

import qcui.util.*;

import qcsim.*;
import qcsim.peer.*;
import qcsim.peer.content.*;
import qcsim.util.*;


public class InitializationView extends TopologyView 
       implements MouseMotionListener
{
  private Vector selectedPeers_;

  private MouseEvent pressStart_;

  private Rectangle selectedArea_;

  public InitializationView(QueryCycleManager manager)
  {
    super(manager, true);
    addMouseMotionListener( this );    

    selectedPeers_ = new Vector();
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent( g );

    if (selectedArea_ != null)
    {
      g.setColor(Color.black);
      g.drawRect((int)selectedArea_.getX(), (int)selectedArea_.getY(), 
                 (int)selectedArea_.getWidth(), 
                 (int)selectedArea_.getHeight());
    }

    // Need to highlight the selected peers.
    for (int i = 0; i < selectedPeers().size(); i++)
    {
      int id = ((Integer)selectedPeers().elementAt(i)).intValue();
      Peer p = peerManager().peer(id);
      p.description().draw(g, Color.yellow);
    }
  } 

  public Vector selectedPeers()
  {
    return selectedPeers_;
  }

  public void clearSelectedPeers()
  {
    selectedPeers_.clear();
  }

  public void mousePressed(MouseEvent e) 
  { 
    super.mousePressed( e );

    pressStart_   = e;
    selectedArea_ = null;
  }

  public void mouseReleased(MouseEvent e) 
  { 
    pressStart_   = null;
    selectedArea_ = null;
    repaint();
  }

  public void mouseDragged(MouseEvent e) 
  {
    if (pressStart_ != null)
    {
      int xpos   = 0;
      int ypos   = 0;
      int width  = 0;
      int height = 0;

      if (e.getX() > pressStart_.getX()) 
      {
        width = e.getX() - pressStart_.getX();
        xpos  = pressStart_.getX();
      }
      else 
      {
        width = pressStart_.getX() - e.getX();
        xpos  = e.getX();
      }

      if (e.getY() > pressStart_.getY())
      {
        height = e.getY() - pressStart_.getY();
        ypos   = pressStart_.getY();
      }
      else
      {
        height = pressStart_.getY() - e.getY();
        ypos   = e.getY();
      }
      selectedArea_ = new Rectangle(xpos, ypos, width, height);

      selectedPeers_.clear();
      // Need to highlight the selected peers.
      for (int i = 0; i < peers(); i++)
      {
        if (selectedArea_.contains(peer(i).description().position()))
          selectedPeers_.addElement(new Integer(peer(i).peerID()));
      }
      repaint();
    } 
  }

  public void mouseMoved(MouseEvent e) {}  
}
