/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui.view;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

import qcui.util.*;

import qcsim.*;
import qcsim.util.*;
import qcsim.peer.*;

public abstract class QueryCycleView extends JComponent
{
  private final Object NULL_NOTIFIEE = new Object();

  private Object notifiee_;

  private QueryCycleManager manager_;

  private JPanel controlPanel_;

  public QueryCycleView(QueryCycleManager manager)
  {
    notifiee_ = NULL_NOTIFIEE;
    manager_ = manager;

    setSize(600, 700);
    setPreferredSize(new Dimension(600, 700));
    setMinimumSize(new Dimension(600, 700));
    setMaximumSize(new Dimension(600, 700));

    setBackground(Color.white);
    setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
    setDoubleBuffered(true);
    setOpaque(true);
    setBackground(Color.white);

    controlPanel_ = new JPanel();
    controlPanel_.setPreferredSize(new Dimension(200, 200));
    controlPanel_.setMinimumSize(new Dimension(200, 200));
    controlPanel_.setMaximumSize(new Dimension(200, 200));
  }

  public abstract String viewName();

  public JPanel controlPanel()
  {
    return controlPanel_;
  }

  public void notifiee(Object notifiee)
  {
    if (notifiee == null)
      notifiee = NULL_NOTIFIEE;
    else
      notifiee_ = notifiee;
  }

  protected void notifyDone()
  {
    try {
      synchronized (notifiee_)
      {
        if (notifiee_ != NULL_NOTIFIEE)
          notifiee_.notify();
      }
    } catch (Exception e) {
      System.err.println("notifyDone exception: "+e);
    }
  }

  protected void clearBackground(Graphics g)
  {
    g.setColor(Color.white);
    g.fillRect(0,0,getWidth(),getHeight());
  }

  protected void drawBackground(Graphics g)
  {
    clearBackground( g );

    if (manager_ == null) return;

    int start = 0;
    int end   = 0;
    int i     = 0;

    // GOOD PEERS
    while(i < peers() &&
          peer(i).behavior().type() < PeerBehavior.HIGHLY_TRUSTED_PEER)  
    {
      end = (int)peer(i).description().position().getX(); 
      i++;
    }
    end += 4; // This is the radius of a peer.

    g.setColor(new Color(255,255,255));
    g.fillRect(start, 0, end-start, getHeight());
    start = end;

    if (i == peers()) return;

    // HIGHLY TRUSTED PEERS
    while(i < peers() && 
          peer(i).behavior().type() == PeerBehavior.HIGHLY_TRUSTED_PEER) 
    {
      end = (int)peer(i).description().position().getX(); 
      i++;
    }
    end += 4; // This is the radius of a peer.

    g.setColor(new Color(224,224,224));
    g.fillRect(start, 0, end-start, getHeight());

    if (i == peers()) return;

    // MALICIOUS PEERS
    g.setColor(new Color(192,192,192));
    g.fillRect(end, 0, getWidth() - end, getHeight());
  }

  protected QueryCycleManager manager()
  {
    return manager_;
  }

  protected PeerManager peerManager()
  {
    return manager_.peerManager();
  }

  protected int peers()
  {
    return peerManager().peers();
  }

  protected Peer peer(int i)
  {
    return peerManager().peer(i);
  }

}
