/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui.view;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.basic.*;

import qcui.util.*;

import qcsim.*;
import qcsim.peer.*;
import qcsim.util.*;

public class ShortestPathView extends QueryCycleView implements ActionListener
{
  private JCheckBox colorTrust_;

  private JLabel clusterCoefficient_;

  private JLabel characteristicPathLength_;

  private JLabel networkDiameter_;

  public ShortestPathView(QueryCycleManager manager, String resource)
  {
    super(manager);
    controlPanel().setLayout(new BorderLayout(0, 50));

    colorTrust_   = new JCheckBox("Color Trust");
    colorTrust_.addActionListener( this );

    JPanel checkBoxPanel = new JPanel();
    checkBoxPanel.setLayout(new BasicOptionPaneUI.ButtonAreaLayout(true, 0));
    checkBoxPanel.add(colorTrust_);

    controlPanel().add(checkBoxPanel, BorderLayout.NORTH);

    JPanel networkDescriptions = new JPanel();
    networkDescriptions.setLayout(new BoxLayout(networkDescriptions, BoxLayout.Y_AXIS));

    clusterCoefficient_ = new JLabel();
    networkDescriptions.add(clusterCoefficient_);

    characteristicPathLength_ = new JLabel();
    networkDescriptions.add(characteristicPathLength_);

    networkDiameter_ = new JLabel();
    networkDescriptions.add(networkDiameter_);

    controlPanel().add(networkDescriptions, BorderLayout.CENTER);

  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent( g );
    super.clearBackground( g );

    controlPanel().setBorder(BorderFactory.createTitledBorder(viewName()));

    Double d  = new Double(manager().peerManager().characteristicPathLength(-1));
    String ds = d.toString();
    if (ds.length() >= 5)
      ds = ds.substring(0,5);

    characteristicPathLength_.setText("Char. Path Length: "+ ds); 

    d  = new Double(manager().peerManager().avgClusterCoefficient());
    ds = d.toString();
    if (ds.length() >= 5)
      ds = ds.substring(0,5);
    clusterCoefficient_.setText("Cluster Coefficient: "+ ds); 

    networkDiameter_.setText("Network Diameter: "+
      manager().peerManager().diameter());

    // DRAW SHORTEST PATH GRAPHS
    Dimension dim = getSize();

    Plot spg = manager().plot(QueryCycleManager.UPLOADS_PATH); 

    int height    = (int) (dim.getHeight() * 0.3);
    int width     = (int) (dim.getWidth() * 0.8);

    int graphY = (int)(dim.getHeight() * 0.35); 
    int graphX = (int)(dim.getWidth() * 0.15);

    spg.graph(g, this, Plot.BAR_GRAPH, 10, 10, 
              graphX, graphY, new Dimension(width, height));


    int autoScale = (int) Math.ceil(spg.maxY() / 10.0);
    height = (int) (dim.getHeight() * 0.4);
    graphY = (int)(dim.getHeight() * 0.90); 
    graphX = (int) peer(0).description().position().getX() - 5; 
    width  = (int) peer(peers()-1).description().position().getX();

    g.setColor(Color.black);
    g.drawLine(graphX,graphY,graphX+width,graphY);   // Draw x axis
    g.drawLine(graphX,graphY,graphX,graphY-height);  // Draw y axis

    Plot sp = manager().plot(QueryCycleManager.SHORTEST_PATH); 
    g.drawString(sp.ylabel(), (int)(dim.getWidth() * 0.25), graphY-height);
    g.drawString(sp.xlabel(), (int)(dim.getWidth() * 0.25), graphY+30);
    
    for (int i = 0; i < sp.plots(); i++)
    {
      double pathLength = sp.plotY(i);

      pathLength = pathLength * (height / 10);
      pathLength = pathLength / autoScale;

      g.setColor(peer(i).description().color(false));
      g.drawLine((int) peer(i).description().position().getX(), graphY+5,
                 (int) peer(i).description().position().getX(), graphY);
      
      g.setColor(peer(i).description().color(colorTrust_.isSelected()));
      g.fillRect((int)peer(i).description().position().getX()-4, 
                 (int)(graphY-pathLength), 8, (int)pathLength);
    }

    g.setFont(new Font("Serif", Font.PLAIN, 10));
    g.setColor(Color.black);
    double scale = 0;
    for (int i = 1; i <= 10; i++)
    {
      scale  = i * autoScale; 
      g.drawString(Integer.toString((int)scale), 
                   graphX-25, (int)(graphY-(i*height/10)+5));
      g.drawLine(graphX, (int)(graphY-(i*height/10)),
                 graphX+5, (int)(graphY-(i*height/10)));
    }
   
    super.notifyDone();
  } 

  public String viewName() 
  { 
    return "Shortest Path View";
  }

  public void actionPerformed(ActionEvent event) 
  {
    validate();
    repaint();
  }

}
