/* ====================================================================
 * The Stanford P2P Sociology Project Software License, Version 1.0
 *
 * Copyright (c) 2003 The Stanford P2P Sociology Project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Stanford P2P Sociology Project(http://p2p.stanford.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * For more
 * information on the Stanford P2P Sociology Project,
 * see <http://p2p.stanford.edu/>.
 *
 */
package qcui.view;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import qcui.util.*;

import qcsim.*;
import qcsim.peer.*;
import qcsim.util.*;

public class TrustView extends QueryCycleView
{
  private JCheckBox colorTrust_;

  private int zoom_;

  public TrustView(QueryCycleManager manager, String resource)
  {
    super(manager);

    zoom_ = 1;

    controlPanel().add(zoomPanel());
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent( g );
    super.drawBackground( g );

    controlPanel().setBorder(BorderFactory.createTitledBorder(viewName()));

    Dimension dim = getSize();

    int height = (int) (dim.getHeight() * 0.85);
    int width  = 0;

    int graphY = (int)(dim.getHeight() * 0.9); 
    int graphX = 0;

    if (peers() == 0)
      return;

    Peer peer = peer(0);
    graphX = (int) peer.description().position().getX() - 5; 

    peer  = peer(peers()-1);
    width = (int) peer.description().position().getX();

    g.setColor(Color.black);
    g.drawLine(graphX,graphY,graphX+width,graphY);   // Draw x axis
    g.drawLine(graphX,graphY,graphX,graphY-height);  // Draw y axis
    
    double trust = 0;
    for (int i = 0; i < peers(); i++)
    {
      trust = peer(i).globalTrust() * zoom_;
      g.setColor(peer(i).description().color(false));
      g.drawLine((int) peer(i).description().position().getX(), graphY+5,
                 (int) peer(i).description().position().getX(), graphY);
      
      g.fillRect((int)peer(i).description().position().getX()-4, 
                 (int)(graphY-trust*height), 8, (int)(trust*height));
    }

    g.setFont(new Font("Serif", Font.PLAIN, 10));
    g.setColor(Color.black);
    double scale = 0;
    String value = null;
    int   length = 0;
    for (int i = 1; i <= 10; i++)
    {
      scale  = (double)i / 10.0;
      value  = Double.toString(scale / zoom_);
      length = (value.length() < 6) ? value.length() : 6;
      g.drawString(value.substring(0,length), 
                   graphX-25, (int)(graphY-(scale*height)+5));
      g.drawLine(graphX, (int)(graphY-(scale*height)),
                 graphX+5, (int)(graphY-(scale*height)));
    }

    super.notifyDone();
  } 

  public String viewName() 
  { 
    return "Trust View";
  }

  private JPanel zoomPanel()
  {
    JPanel  zoom   = new JPanel();
    JSlider slider = new JSlider(JSlider.HORIZONTAL,1,10,1);

    //Turn on labels at major tick marks.
    slider.setMajorTickSpacing(1);
    slider.setMinorTickSpacing(1);
    slider.setPaintTicks(true);
    slider.setPaintLabels(true);

    ChangeListener listener = new ChangeListener() {
      public void stateChanged(ChangeEvent event) 
      {
        JSlider slider = (JSlider) event.getSource();
        zoom_  = slider.getValue();
        repaint();
      }
    };
    slider.addChangeListener( listener );

    zoom.setLayout(new BoxLayout(zoom,BoxLayout.Y_AXIS));
    zoom.add(new JLabel(viewName() + " Zoom Control"));
    zoom.add(slider);

    zoom.setPreferredSize(new Dimension(175, 100));
    zoom.setMinimumSize(new Dimension(175, 100));
    zoom.setMaximumSize(new Dimension(175, 100));
    return zoom;
  }

}
